########################################################################
# Copyright (c) 2003 Trimble Navigation Limited
# $Source: /home/CVS/fireballTop/fireballFS/root/usr/local/apache/htdocs/fireball/perl-scripts/Fireball.pm,v $
# $Id: Fireball.pm,v 1.8.2.2 2003/11/05 01:31:53 tom Exp $
########################################################################

########################################################################
# This file contains the Fireball CGI perl module.  It contains
# a few procedures that can be used by other Perl CGI scripts to assist
# in generating Fireball web pages.  Using these scripts ensures that
# the CGI pages have a uniform look and feel.
#
# To use this library, a CGI script must include the library with the 
# command :
#
#   use Fireball ;
#
# If this will work if the script is located in the same directory
# as this module (/usr/local/apache/htdocs/fireball/perl-scripts).
# If not, then the script must tell Perl where the library is located
# by using something like:
#
#   use lib ".." ;
#   use Fireball ;
#
########################################################################

package Fireball ;
require Exporter ;

our @ISA = qw( Exporter ) ;
our @EXPORT = qw( titleWithHelp 
                  startPage 
                  FBstart_html 
                  FBheader 
                  timestamp 
                  @dayStrings 
                  @monthStrings 
                  IsRpmFile
                  underWarranty 
                  RpmVerify
                  readFTPSetup
                  generateFtppasswd
                  regenerateFtpAccess
                  getGuiSecurity
                  updateGuiSecurity
                  makepw
                  updateGuiAccounts
                  getGuiAccounts
                  getAntennaInfo
                  getAntennaSetup
                  updateAntennaSetup
                  getEthernetSetup
                  updateEthernetSetup
                  getSessionData
                  updateSessionData
                  EnaDisSession
                  getPortSetup
                  saveGpssdConfig
                  saveMetConfig
                  savePppConfig
                  sendBinaryFile 
                  getRefStationData 
                  updateRefStationData 
                  getNtpServers
                  saveNtpServers
                  getIoStatus
                ) ;

use CGI qw/:standard/ ;
use IO::Socket;


@dayStrings = ("Sun","Mon","Tue","Wed","Thu","Fri","Sat") ;
@monthStrings = ("Jan","Feb","Mar","Apr","May","Jun",
                    "Jul","Aug","Sep","Oct","Nov","Dec") ;

########################################################################
sub FBheader
########################################################################
# Puts out the standard Fireball Header, with a page expiration date.
# No Parameters expected.
# We want all of our CGI-generated web pages to always be re-submitted
# on each request, without any caching in the user's browser.  To do
# this, we set the return documents to expire ten years ago (-10y).
# You would think that just setting that to +1second would be adequate,
# but this ignores the problem of the user's computer not matching our
# receiver's time.  Web page expiration times are tagged at the source
# (inside Fireball) but the expiration check is done at the browser,
# using a potentially completely different time scale.  A poorly set
# clock in a PC could cause our pages not to refresh properly.
########################################################################
{
  print CGI::header(-expires=>"-10y") ;
} #end of FBheader()



########################################################################
sub FBstart_html
########################################################################
# Starts an html page with our background color, a title, etc.
# Passed a parameter for an HTML page name which might be visible
# somewhere in the browser, but probably not.
########################################################################
{
  print CGI::start_html( 
          -bgcolor=>'#fff5e4',
          -title=>$_[0],
          -head=>meta( {-http_equiv=>'Content-Type',
                        -content=>'text/html; charset=iso-8859-1'
                       } 
                     )
                .meta( {-name=>'copyright',
                        -content=>
                               'Copyright(c) 2003 Trimble Navigation Ltd.'
                       } 
                     )
                  ), "\n" ;
} #end of FBstart_html()



########################################################################
sub startPage
########################################################################
# Combines FBheader and FBstart_html into a single call.  This is
# a very common idiom in simple CGI files that don't put up a title
# or help button.  Requires the page name as the parameter.
########################################################################
{
  FBheader() ;
  FBstart_html( $_[0] ) ;
}



########################################################################
sub titleWithHelp
########################################################################
{
  startPage( $_[0] ) ;
  
  # The screen title and the help button.  Put in a table to get a nice
  # layout along the top of the screen.  When the image button is
  # clicked, calls the showHelp() function to display the desired help
  # page.  By returning false, the onClick handler suppresses actually
  # following the link, so no link data is needed.

  print table( {width=>"100%"},
          Tr(
             td( h2( $_[0] ) ),
             $_[1] 
             ? td( {align=>'right'}, 
                 a( {href=>"" ,
                    onMouseOver=>'document.images[0].src  = "/images/helpRollover.gif"',
                    onMouseOut =>'document.images[0].src  = "/images/help.gif"',
                    onClick=>
                       "top.showHelp('$_[1]') ;
                        return false"},
                    img( { src=>"/images/help.gif",
                           border=>"0",
                         } 
                       ) 
                  ) 
                 )
              : td( '&nbsp;' )
            ) 
        );

} #end of titleWithHelp()



########################################################################
sub timestamp
########################################################################
# Create Timestamp of the form:
#   UTC Date & Time: 16 Sep 2001 - Thu of GPS Week 1130 - 17:46:52 
# Returns this as a string.
########################################################################
{
  my $time=time() ; 

  my ($s,$m,$h,$md,$mon,$y,$wday) = gmtime($time) ;

  # Determine the GPS week by subtracting the absolute time of
  # beginning of 1980/Jan/06 from current time and dividing by seconds
  # in a week.
  my $gpsweek = int( ($time - 315964800)/604800 ) ;

  sprintf( "<b>UTC Date & Time: </b>"
                 ."%d %s %d - %s of GPS Week %d - %02d:%02d:%02d",
                 $md, $monthStrings[$mon], $y+1900,
                 $dayStrings[$wday], $gpsweek, $h,$m, $s ) ;
} # end of timestamp()



########################################################################
sub RpmVerify           
########################################################################
# Called to verify the integrity of an RPM file.  
# $_[0] is the full pathname of the file in /var/gps/incoming.
# $_[1] is true for verbose and false for non-verbose.
#       Defaults to true if unspecified.
#       If verbose, HTML code is emitted to stdout that shows the 
#       steps and actions done during the verification.
#
# Returns an empty string if the file is okay.
# Otherwise returns an error message of some sort.
#
# Reclaims space in /var/ partition in order to find sufficient space
# for RPM to make a copy of the original file.
#
# This function is called in at least three circumstances:
#   - When a new firmware file is uploaded via the GUI, we verify its 
#     contents.
#   - When a Programmatic "Verify FirmwareFile" command is received.
#   - When an install request is made, (either GUI or programmatic) the
#     selected firmware file is verified before being installed.
########################################################################
{
  my $filePath = $_[0] ;

  my $verbose = 1 ;
  $verbose = $_[1] if ($_[1]) ;

  my $fileSize = (-s $filePath) ;

  # Determine if there is enough space in the /var/tmp area.
  my $availSpace = (split " ", qx( df /var/tmp ))[10] * 1024 ;

  if ( ($fileSize+100000) >  $availSpace )
  {
    if ($verbose)
    {
      print '<br>Insufficient file space in /var/tmp.' ;
      print 
       "<br>RPM size : $fileSize<br>Available Space : $availSpace<br>" ;
    }

    # Here we need to try to reclaim some space in /var.
    # We can remove or prune files in /var/log
    # We can remove files in /var/tmp

    # Remove any rotated logs (ending in .1, .2, ...)
    for my $f ( sort {(-s $b) <=> (-s $a)} glob( '/var/log/*.[1-9]' ) )
    {
      print "Reclaiming log file $f : ", (-s $f), br() if ($verbose) ;
      system( "/usr/bin/logger -t FirmwareVerify 'Reclaiming $f'" ) ;
      system( "/usr/bin/sudo rm $f" ) ;
      $availSpace = (split " ", qx( df /var/tmp ))[10] * 1024 ;
      if ( ($fileSize+100000) <  $availSpace ) 
      {
        print "Available Space up to $availSpace<p>" if ($verbose) ;
        goto TEST ;
      }
    }

    # That wasn't enough.  Try removing files in /var/tmp
    for my $f ( sort {(-s $b) <=> (-s $a)} glob( '/var/tmp/*' ) )
    {
      print "Reclaiming log file $f : ", (-s $f), br() if ($verbose) ;
      system( "/usr/bin/logger -t FirmwareVerify 'Reclaiming $f'" ) ;
      system( "/usr/bin/sudo rm $f" ) ;
      $availSpace = (split " ", qx( df /var/tmp ))[10] * 1024 ;
      if ( ($fileSize+100000) <  $availSpace ) 
      {
        print "Available Space up to $availSpace<p>" if ($verbose) ;
        goto TEST ;
      }
    }

    # That wasn't enough.  Try zeroing out some big log files.
    for my $f ( sort {(-s $b) <=> (-s $a)} glob( '/var/log/*' ) )
    {
      print "Pruning log file $f : ", (-s $f), br() if ($verbose) ;
      system( "/usr/bin/logger -t FirmwareVerify 'Pruning $f'" ) ;
      open F, '>/tmp/emptyFile' ;
      print F qx( /bin/date ) ;
      print F "Log file pruned during firmware verify.\n" ;
      close( F ) ;
      system( "/usr/bin/sudo cp /tmp/emptyFile $f" ) ;

      $availSpace = (split " ", qx( df /var/tmp ))[10] * 1024 ;
      if ( ($fileSize+100000) <  $availSpace ) 
      {
        print "Available Space up to $availSpace<p>" if ($verbose) ;
        goto TEST ;
      }
    }

    return 'Insufficient /var/tmp space' ;
  }

TEST:
  # Run the RPM verification program.
  # The results on a successful test is a string containing 'md5 OK'.
  # If that's missing then the results are regarded as an error message.
  my $testResults = qx( /usr/lib/rpm/rpmk -K $filePath 2>&1 ) ;
  if ($testResults =~ 'md5 OK')
  {
    return '' ;
  }
  else
  {
    return $testResults ;
  }
} # end of RpmVerify()



########################################################################
sub IsRpmFile
########################################################################
# Check if a file is really an RPM file.  
# Parameter $_[0] is the filename.
#
# It's first four
# characters should contain the magic number sequence
# 0xed 0xab 0xee 0xdb
# This subroutine returns true if the characters match, and false
# otherwise.
########################################################################
{
  my $filepath = $_[0] ;

  my ( $c1,$c2, $c3, $c4 ) ;
  if (open( RPMFILE, $filepath ) )
  {
    $c0 = ord( getc( RPMFILE ) ) ;
    $c1 = ord( getc( RPMFILE ) ) ;
    $c2 = ord( getc( RPMFILE ) ) ;
    $c3 = ord( getc( RPMFILE ) ) ;
    close( RPMFILE ) ;
  }

  return ( $c0 == 0xed
        && $c1 == 0xab
        && $c2 == 0xee
        && $c3 == 0xdb
         ) ;

} # end of IsRpmFile



########################################################################
sub underWarranty
########################################################################
# Verify that a specified firmwarefile can be installed under the
# warranty date installed on this system.
# $_[0] is the path name of the firmware file.
# $_[1] is the verbose flag : if true, emits information to stdout,
#       suitable for an HTML Page.  If false, only a return code
#       is generated.
# $_[2] is a reference to a string in which is returned the
#       contents of the RPM file's description info.  The idea here
#       is that since the Warranty has to look this up, we don't
#       want to look it up twice.  It is up to the caller whether this
#       information is preserved, which will only occur if this
#       reference is passed in.
#
# Returns true (1) if the firmware file can be installed.
#         false (0) if not.
#
########################################################################
{
  my $filepath = $_[0] ;
  my $verbose  = $_[1] ;

  # This parameter may be empty, if we don't want the description
  # returned via a reference.
  my $descriptionReference = $_[2] ;


  #print "Filepath : $filepath\n" ;
  #print "Verbose  : $verbose\n" ;
  #print "DescRef  : $descriptionReference\n" ;

  # warranty -w returns:
  #   war date :03312003
  my $warrantyDate = qx(/usr/bin/sudo /usr/local/fireball/warranty -w) ;
  $warrantyDate = (split("[ :]+",$warrantyDate))[2] ;

  # WarrantyDate is converted to format 2003/06/20 for comparison to the
  # firmware date from an RPM package (with leading zeroes on all
  # quantities).
  if ($warrantyDate != 0)
  {
    my ($month,$day,$year) = ($warrantyDate =~ /(..)(..)(....)/) ;
    $warrantyDate = sprintf( "%04d/%02d/%02d", $year,$month,$day ) ;
  }
  else
  {
    $warrantyDate = '2000/01/01' ;
  }

  # Get the descriptive paragraph info from the RPM package.
  my $description = 
          qx( /usr/lib/rpm/rpmq -q -p $filepath --qf '%{DESCRIPTION}') ;

  # If the $descriptionReference (third parameter) is defined,
  # then it should be a reference to a string in which the caller
  # wants the RPM description returned.
  if ($descriptionReference)
  {
    # Copy the description to the string pointed to by the reference.
    $$descriptionReference = $description ;
  }

  # Extract the firmware date from the description data.  It will be in
  # a line looking like: "Firmware Date 2003/06/20".  We just want the
  # '2003/06/20' section.
  my $firmwareDate = ($description =~ m/.*Firmware Date\s+(\S+)/ )[0] ;

  # Adjust firmwareDate to match the expected format.  This is to
  # compensate for the tendency of people who type in the date to
  # not follow a strict format, especially by dropping leading zeroes
  # on month or day.
  my ($year,$month,$day) = ($firmwareDate =~ m|(.+)/(.+)/(.+)|) ;
  $firmwareDate = sprintf( "%04d/%02d/%02d", $year,$month,$day ) ;

  if ($verbose)
  {
    print table( Tr( td( {align=>'right'}, 
                         "Warranty Expiration Date:" 
                       ),
                     td( $warrantyDate ) 
                   ),

                 Tr( td( {align=>'right'}, "Firmware Date:" ),
                     td( $firmwareDate ) 
                   ),
               ) ;
  }

  # Based on a string compare, we decide if this firmware is
  # installable.  This works because we have identical formats and 
  # they have the numeric data with most significant stuff first.
  return ($warrantyDate ge $firmwareDate )

} # end of underWarranty()


########################################################################
sub readFTPSetup
########################################################################
# Utility function for setting a bunch of variables to indicate the
# current FTP setup.  The return values are sent back via 
# reference-parameters
# $_[0] = reference to $anonEnabled
# $_[1] = reference to $namedEnabled
# $_[2] = reference to $adminEnabled
# $_[3] = reference to $anonDeleting
# $_[4] = reference to $namedDeleting
# $_[5] = reference to $namedPassword  (encrypted)
# $_[6] = reference to $adminPassword  (encrypted)
########################################################################
{

  # Read in the contents of /etc/ftpaccess to determine current setup
  # The variabilities here are best understood by examining
  # /etc/ftpaccess, which is well documented.  Look for "GUI WILL EDIT" to
  # find the relevant lines.
  my $anonEnabled = 0 ;
  my $namedEnabled = 0 ;
  my $adminEnabled = 0 ;
  my $anonDeleting = 0 ;
  my $namedDeleting = 0 ;
  my $namedPassword = '' ;
  my $adminPassword = '' ;

  if ( open( FTPCONF, '/etc/ftpaccess' ) )
  {
    #print "<PRE>" ;
    while (<FTPCONF>)
    {
      #print $_ ;
      if (m/^#?\s*class\s+AnonClass\s+anonymous/)
      {
        # The line "class AnonClass anonymous" is either enabled or
        # disabled by the presence of a '#' in the first column.
        #print $_ ;
        if (! m/^#/)
        {
          $anonEnabled = 1 ;
        }
      }

      if (m/^#?\s*allow-uid\s+adminFTP/)
      {
        # The line "allow-uid adminFTP" is either enabled or
        # disabled by the presence of a '#' in the first column.
        #print $_ ;
        if (! m/^#/)
        {
          $adminEnabled = 1 ;
        }
      }

      if (m/^#?\s*allow-uid\s+netrsFTP/)
      {
        # The line "allow-uid netrsFTP" is either enabled or
        # disabled by the presence of a '#' in the first column.
        #print $_ ;
        if (! m/^#/)
        {
          $namedEnabled = 1 ;
        }
      }

      if (m/^delete\s+(no|yes)\s+anonymous/)
      {
        # anonymous file deletion is controlled by a "yes" or "no"
        # field in this line.
        #print $_ ;
        if (m/yes/)
        {
          $anonDeleting = 1 ;
        }
      }

      if (m/^delete\s+(no|yes)\s+guest/)
      {
        # netrsFTP file deletion is controlled by a "yes" or "no"
        # field in this line.
        #print $_ ;
        if (m/yes/)
        {
          $namedDeleting = 1 ;
        }
      }
    }
    close( FTPCONF ) ;
    #print "</PRE>" ;
  }

  $anonDeleting = 0 unless ($anonEnabled) ;
  $namedDeleting = 0 unless ($namedEnabled) ;


  if (open( PW, '/etc/sysconfig/ftppasswd' ) )
  {
    while (<PW>)
    {
      $adminPassword = $1 if ( m/^adminFTP:([^:]+):/ ) ;
      $namedPassword = $1 if ( m/^netrsFTP:([^:]+):/ ) ;
    }
    close( PW ) ;
  }



  ${$_[0]} = $anonEnabled ;
  ${$_[1]} = $namedEnabled ;
  ${$_[2]} = $adminEnabled ;
  ${$_[3]} = $anonDeleting ;
  ${$_[4]} = $namedDeleting ;
  ${$_[5]} = $namedPassword ;
  ${$_[6]} = $adminPassword ;

} # end of readFTPSetup()




########################################################################
sub generateFtppasswd
########################################################################
#
# Regenerate the file /etc/sysconfig/ftppasswd
#  $_[0] = the named account password (for account netrsFTP)
#  $_[1] = the admin account password (for account adminFTP)
#  $_[2] = the cleartext namedpassword.  Encrypted if set.
#  $_[3] = the cleartext adminpassword,  Encrypted if set.
# If an encrypted password is set blank, then no entry will end up in
# the ftppasswd file.  Thus, an existing account can be deleted by
# specifying a null password.
#
########################################################################
{
  my $netrspw = $_[0] ;
  my $adminpw = $_[1] ;
  my $netrsClearPW = $_[2] ;
  my $adminClearPW = $_[3] ;

  # Encrypt any plaintext passwords that were supplied.
  if ($netrsClearPW)
  {
    my $seed = join '', 
              ('.','/','0'..'9','A'..'Z','a'..'z')[rand 64, rand 64] ;
    $netrspw = crypt( $netrsClearPW, $seed ) ;
  }

  if ($adminClearPW)
  {
    my $seed = join '', 
              ('.','/','0'..'9','A'..'Z','a'..'z')[rand 64, rand 64] ;
    $adminpw = crypt( $adminClearPW, $seed ) ;
  }


  open( NEWPW, '>/tmp/ftppasswd' ) ;

  if ($adminpw)
  {
    print NEWPW 'adminFTP:',
                $adminpw,
                ":5002:5000::/var/gps/incoming:/bin/date\n" ;
  }
  
  if ($netrspw)
  {
    print NEWPW 'netrsFTP:',
                $netrspw,
                ":5001:5000::/var/gps:/bin/date\n" ;
  }
  
  close( NEWPW ) ;

  # Copy the new passwd file into the final location.
  system( 'sudo cp /tmp/ftppasswd /etc/sysconfig/ftppasswd' ) ;

  # Merge the new password data into the system file /etc/passwd
  system( 'sudo /usr/local/fireball/mergePasswds' ) ;

} # end of generateFtppasswd()



########################################################################
sub regenerateFtpAccess
########################################################################
# Regenerates the wu-ftp configuration file /etc/sysconfig/ftpaccess.
# This file controls which accounts are enabled and with what
# capabilities.  Once the file is updated, the FTP daemon is restarted
# to force it to use the new configuration.
#
# Parameters that come into this script are:
#   $_[0] = anonEnabled
#   $_[1] = anonDeleting
#   $_[2] = namedEnabled
#   $_[3] = namedDeleting
#   $_[4] = adminEnabled
#
# The file is completely regenerated from the default file, 
# /usr/local/fireball/defaults/sysconfig/ftpaccess, whenever this
# subroutine is called.  Five specific lines are rewritten based on the
# five parameters provided.
# Note that this subroutine ignores any previous changes made to the
# /etc/syconfig/ftpaccess file, and always starts from scratch with the
# default file.  Although this does prevent some forms of "hacking" on 
# the file, it also protects us against poorly done hacks, or a missing
# sysconfig file.
#
########################################################################
{
  my $anonEnabled    = $_[0] ;
  my $anonDeleting   = $_[1] ;
  my $namedEnabled   = $_[2] ;
  my $namedDeleting  = $_[3] ;
  my $adminEnabled   = $_[4] ;

  # Update the configuration in /etc/ftpaccess with the new data.
  # Generate a new copy of the file by reading through the original 
  # file, transfering all lines, and adjusting those specific lines
  # that are under GUI control.
  open( OLDCONF, '/usr/local/fireball/defaults/sysconfig/ftpaccess' ) ;
  open( NEWCONF, '>/tmp/ftpaccess' ) ;
  
  while (<OLDCONF>)
  {
    if (m/^#?class\s+AnonClass\s+anonymous/)
    {
      # The line "class AnonClass anonymous" is either enabled or
      # disabled by the presence of a '#' in the first column.
      print NEWCONF '#' if ( ! $anonEnabled ) ;
      print NEWCONF "class   AnonClass    anonymous  *\n" ;
      next ;
    }

    if (m/^#?\s*allow-uid\s+adminFTP/)
    {
      # The line "allow-uid adminFTP" is either enabled or
      # disabled by the presence of a '#' in the first column.
      print NEWCONF '#' if ( ! $adminEnabled ) ;
      print NEWCONF "allow-uid adminFTP\n" ;
      next ;
    }

    if (m/^#?\s*allow-uid\s+netr[sx]FTP/)
    {
      # The line "allow-uid netrsFTP" is either enabled or
      # disabled by the presence of a '#' in the first column.
      # The test above allows rewriting either "netrsFTP" or "netrxFTP"
      # for backward compatibility to older firmware.  See the CVS logs
      # for v1.17 of this file for details.
      print NEWCONF '#' if ( ! $namedEnabled ) ;
      print NEWCONF "allow-uid netrsFTP\n" ;
      next ;
    }

    if (m/^delete\s+(no|yes)\s+anonymous/)
    {
      # anonymous file deletion is controlled by a "yes" or "no"
      # field in this line.
      print NEWCONF 'delete ',
                    $anonDeleting ? 'yes' : 'no',
                    " anonymous\n"  ;
      next ;
    }

    if (m/^delete\s+(no|yes)\s+guest/)
    {
      # netrsFTP file deletion is controlled by a "yes" or "no"
      # field in this line.
      print NEWCONF 'delete ',
                    $namedDeleting ? 'yes' : 'no',
                    " guest\n"  ;
      next ;
    }

    print NEWCONF $_ ;
  }
  
  close( NEWCONF ) ;
  close( OLDCONF ) ;

  # Copy the new configuration file into the real config directory.
  # Must use sudo as sysconfig is writeable only by root.
  system( 'sudo cp /tmp/ftpaccess /etc/ftpaccess' ) ;

  # Restart the FTP daemon, so that it will re-read its configuration.
  # The daemon is started and monitored by the frontpaneltask.  So
  # it is sufficient to just kill the existing daemon.  Frontpaneltask
  # will notice and restart it.
  if ( -r '/var/run/ftpd.pid' ) 
  {
    my $pid = qx( cat /var/run/ftpd.pid ) ;
    #print "Killing pid $pid" ;
    system( "/usr/bin/sudo /bin/kill $pid" ) ;
  }

} # end of regenerateFtpAccess()



########################################################################
sub getGuiSecurity
########################################################################
# Returns the security restrictions on the GUI interface, as defined
# in three htaccess files.
# Passed a reference to a hash, in which the results are returned.
# %{$_[0]} 
# The hash will have a value for 'sysdmin', 'gpsadmin' and 'fileaccess'.
# Each value will be either true (restricted) or false (unrestricted).
#
########################################################################
#
{
  my $restrictions = $_[0] ; # A reference to a hash.

  # this is a list of security types.  Each of them has a
  # matching htaccess.type file.
  my @levels = ( 'sysadmin', 'gpsadmin', 'fileaccess' ) ;

  foreach my $r (@levels)
  {
    # Default this level to unrestricted, unless we find out different.
    ${$restrictions}{ $r } = 0 ;

    if (open( HTACCESS, "/etc/sysconfig/htaccess.$r" ) )
    {
        #print '<PRE>' ;
        #print "$r\n" ;
        while (<HTACCESS>) 
        {
          if ( $_ =~ '^require ')
          {
            ${$restrictions}{ $r } = 1 ;
          }
        } 
        
        #print '</PRE>' ;
        close HTACCESS ;
    }
  }
} # end of getGuiSecurity()



########################################################################
sub updateGuiSecurity
########################################################################
# Updates the security restrictions on the GUI interface, as defined
# in three htaccess files.
# Passed a reference to a hash, in which the new security levels are
# encoded.
# %{$_[0]} 
# The hash will have a value for 'sysdmin', 'gpsadmin' and 'fileaccess'.
# Each value will be either true (restricted) or false (unrestricted).
# The htaccess file matching each permission is updated to the
# new restrictions.
#
# Returns true if anything was changed.
########################################################################
{
  my $newRestrictions = $_[0] ; # A reference to a hash.

  my $changeMade = 0 ; # true if a change is made to a file.

  # Get the current restrictions so we can determine what needs
  # changing.
  my %oldRestrictions ;
  getGuiSecurity( \%oldRestrictions ) ;

  # this is a list of security types.  Each of them has a
  # matching htaccess.type file.
  my @levels = ( 'sysadmin', 'gpsadmin', 'fileaccess' ) ;

  foreach my $r (@levels)
  {
    if (${$newRestrictions}{$r} != $oldRestrictions{$r})
    {
      my $tmpFile = "/tmp/htaccess.$r" ;

      open( HTACCESS, ">$tmpFile" ) ;
      print HTACCESS << "EOF" ;
AuthType Basic
AuthUserFile /etc/sysconfig/htpasswd
AuthGroupFile /etc/sysconfig/htgroups
EOF

      print HTACCESS "AuthName " ;
      if ($r eq 'sysadmin')     { print HTACCESS "SysAdmin\n" }
      elsif ($r eq 'gpsadmin')  { print HTACCESS "GpsAdmin\n" }
      else                      { print HTACCESS "FileAccess\n" }

      print HTACCESS "require group $r\n" if (${$newRestrictions}{$r}) ;
      close HTACCESS ;

      # Install the new file.
      system "sudo cp $tmpFile /etc/sysconfig/" ;
      unlink $tmpFile ;
    }
  }

  return $changeMade ;

} # end of updateGuiSecurity()



########################################################################
sub getGuiAccounts
########################################################################
# Returns a hash containing the current GUI account state.
# Hash is returned via a hash-reference passed in via $_[0].
# Keys to the has are the account names.
# Values of the hash are sub-hashes
#   { 'password' => encrypted password string, 13 characters,
#     'sysadmin' => boolean indicating sysadmin privileges
#     'gpsadmin' => boolean indicating gpsadmin privileges
#     'fileaccess' => boolean indicating fileaccess privileges
#   }
#
########################################################################
{
  my $accountHashReference = $_[0] ;

  # Open and read the current account configuration state.
  my %accounts ;

  if ( open( HTPASSWD, '/etc/sysconfig/htpasswd' ) )
  {
    # Parse the data in htpasswd 
    while (<HTPASSWD>)
    {
      chomp ; # Drop newline
      my ($name,$pw) = split ':' ;
      $accounts{$name}{'password'} = $pw ;
      $accounts{$name}{'sysadmin'} = 0 ;
      $accounts{$name}{'gpsadmin'} = 0 ;
      $accounts{$name}{'fileaccess'} = 0 ;
    }
    close( HTPASSWD ) ;
  }

  # Read in the group information from htgroups.
  # Each line contains a group name and a list of members.
  if ( open( HTGROUPS, '/etc/sysconfig/htgroups' ) )
  {
    while (<HTGROUPS>)
    {
      chomp ; # Drop newline
      if ( $_ =~ '^sysadmin:')
      {
        my @names = split / / ;
        shift( @names ) ;
        for my $name (@names)
        {
          $accounts{$name}{'sysadmin'} = 1 ;
        }
      }
      elsif ($_ =~ '^gpsadmin:')
      {
        my @names = split / / ;
        shift( @names ) ;
        for my $name (@names)
        {
          $accounts{$name}{'gpsadmin'} = 1 ;
        }
      }
      elsif ($_ =~ '^fileaccess:')
      {
        my @names = split / / ;
        shift( @names ) ;
        for my $name (@names)
        {
          $accounts{$name}{'fileaccess'} = 1 ;
        }
      }
    }
    close( HTGROUPS ) ;
  }

  # Copy the created Hash into the location given by the 
  # parametric reference.
  %{$accountHashReference} = %accounts ;

} # end of getGuiAccounts()


########################################################################
sub updateGuiAccounts
########################################################################
# Updates the information in the configuration files defining all 
# GUI accounts.
# The new data is fed to this subroutine via a hash, identical in
# form to the one produced by getGuiAccounts.
# Keys to the has are the account names.
# Values of the hash are sub-hashes
#   { 'password' => encrypted password string, 13 characters,
#     'sysadmin' => boolean indicating sysadmin privileges
#     'gpsadmin' => boolean indicating gpsadmin privileges
#     'fileaccess' => boolean indicating fileaccess privileges
#   }
########################################################################
{
  my $accountsReference = $_[0] ;

  # Update the etc/sysconfig/htpasswd which defines the account names
  # and their encrypted passwords.
  # Each line looks like:
  #     acctname:encryptedpasswd
  # The file is first created in /tmp, and is then copied into
  # the final location to preserve ownership and permissions.
  # Sudo is needed because the file isn't writeable by the user running
  # the apache daemon.
  open( HTPASSWD, ">/tmp/htpasswd" ) ;
  foreach ( sort keys %{$accountsReference} )
  {
    print HTPASSWD "$_:${$accountsReference}{$_}{'password'}\n" ; 
  }
  close HTPASSWD ;


  # Update the /etc/sysconfig/htgroups file which defines which
  # accounts have access rights in which groups.
  # Same sort of tmp files as for htpasswd above.
  # File has format:
  #     groupname: user1 user2 ...
  # We have three groups to define, sysadmin, gpsadmin, fileaccess. 
  open( HTGROUPS, ">/tmp/htgroups" ) ;

  for my $group ( 'sysadmin', 'gpsadmin', 'fileaccess' )
  {
    print HTGROUPS "$group:" ;
    foreach ( sort keys %{$accountsReference} )
    {
      print HTGROUPS " $_" if ${$accountsReference}{$_}{$group} ;  
    }
    print HTGROUPS "\n" ;
  }
  close HTGROUPS ;

  system( 'sudo cp /tmp/htpasswd /etc/sysconfig/htpasswd' ) ;
  system( 'sudo cp /tmp/htgroups /etc/sysconfig/htgroups' ) ;
  unlink( '/tmp/htpasswd' ) ;
  unlink( '/tmp/htgroups' ) ;

} # end of updateGuiAccounts()



########################################################################
sub makepw
########################################################################
# subroutine makepw()
#   Passed an ascii password in $_[0]. 
#   Returns an encrypted password.
########################################################################
{
  # Generate a random seed value by picking two random characters and
  # then joining them together into a 2-character string.
  my $seed=join '', 
           ('.','/','0'..'9','A'..'Z','a'..'z')[rand 64, rand 64];
  return crypt( $_[0], $seed ) ;
} # End of subroutine makepw()




########################################################################
sub getAntennaInfo
########################################################################
# Read the anteanna database from the file
#
#       /usr/local/fireball/antennaModel.config.
#
# From this we parse the possible antenna types and a bunch of
# information about each type.  The values determined are saved
# in a hash structure, indexed by the AntType, which is a simple
# numeric string.  
# This hash structure is returned to the caller via a reference-to-hash
# parameter, $_[0].
# Note that the keys to the hash are "numeric".  Unclear how Perl
# actually treats them, but $antInfo{ 1 } probably is equivalent to
# $antInfo{ '1' }, and is not the same as $antInfo{ '01' }.  So be sure
# to use a true integer values for all keys.
# $antInfo{ key } contains a bunch of sub-hashes or sub-arrays.
#   $antInfo{type}{'name'} = Trimble name for the antenna type.
#   $antInfo{type}{'code'} = Trimble two-chracter code for antenna type.
#   $antInfo{type}{'ngs'}  = NGS/IGD string name for antenna type.
#                            Up to 20 characters.  May includes spaces.
#   $antInfo{type}{'methodCount'}  = How many antenna refpoint methods
#                            are defined for this antenna.  Methods are
#                            indexes 0 .. methodCount-1.
#   $antInfo{type}{'defMethod'} = The default method index.
#   $antInfo{type}{'methods'}[n] = an array of method strings.
#
########################################################################
{
  my $refToHash = $_[0] ; # Reference to the place to return the data

  my %antInfo ; # We'll build up our data here, and then return it.

  open( ANTINFO, '/usr/local/fireball/antennaModel.config' ) ;
  #print "<pre>" ;
  while ( <ANTINFO> )
  {
    next if (m/^\s*$/) ;
    if (m/^AntName:\s*(.*)$/)
    {
      # First line is "AntName:  Name Of Antenna"
      my $name = $1 ;
      $name =~ s/\s+$// ; # drop any trailing spaces

      my $antType = <ANTINFO> ;
      $antType =~ s/^.*:\s*(\d+)\s*$/$1/ ;
      $antInfo{$antType}{'name'} = $name ;

      my $NGS = <ANTINFO> ;
      $NGS =~ s/^.*:\s*(.*)$/$1/ ;
      chomp $NGS ;
      $antInfo{$antType}{'ngs'} = $NGS ;

      my $charCode = <ANTINFO> ;
      $charCode =~ s/^.*:\s*(.*)$/$1/ ;
      chomp $charCode ;
      $antInfo{$antType}{'code'} = $charCode ;

      my $RinexMethod = <ANTINFO> ;
      #$RinexMethod =~ s/^.*:\s*(.*)$/$1/ ;
      #chomp $RinexMethod ;
      #$antInfo{$antType}{'rinexMethod'} = $RinexMethod ;

      my $RinexOffset = <ANTINFO> ;
      #$RinexOffset =~ s/^.*:\s*(.*)$/$1/ ;
      #chomp $RinexOffset ;
      #$antInfo{$antType}{'rinexOffset'} = $RinexOffset ;

      my $methodCount = <ANTINFO> ;
      $methodCount =~ s/^.*:\s*(\d+)\s*$/$1/ ;
      ++$methodCount ;
      $antInfo{$antType}{'methodCount'} = $methodCount ;

      my $defMethod = <ANTINFO> ;
      chomp $defMethod ;
      $defMethod =~ s/^.*\[(\d+)\].*/$1/ ;
      $antInfo{$antType}{'defMethod'} = $defMethod+1 ;

      #print "$name, $antType, $NGS, $methodCount, $defMethod\n" ;

      for (my $m = 0 ; $m < $methodCount ; ++$m)
      {
        my $method = <ANTINFO> ;
        $method =~ s/^.*:\s*(.*)$/$1/ ;
        chomp $method ;
        $method =~ s/\s+$// ; # drop any trailing spaces

        $antInfo{$antType}{'methods'}[$m] = $method ;
        
        # Drop the next line of method info for now.
        $_ = <ANTINFO> ;

        #print "--",$methods{$antType}{$m},"--\n" ;
      }
    }
  }
  close( ANTINFO ) ;

  # Return the parsed data hash to the caller via the reference
  # parameter.
  %{$refToHash} = %antInfo ;

} # end of getAntennaInfo()



########################################################################
sub getAntennaSetup
########################################################################
# Returns the details of the current antenna setup.
# The information is extracted from the 'sysinfo -p ref_stn' output.
# Several elements are parsed into a hash, keyed by specific names.
# The hash is then returned to the caller via a reference parameter.
# Hash elements that are set:
#   type   : numeric antenna type.
#   serial : Serial number string.
#   height : antenna height in meters.
#   method : numeric index into refpoint method table.  0..n
########################################################################
{
  my $refToHash = $_[0] ;
  my %antSetup ;

  # Read in the current antenna settings from the sysinfo system.
  open( REFCONF, 'sudo /usr/local/fireball/sysinfo -p ref_stn |' ) ;
  while (<REFCONF>)
  {
    if (m/^\s*AntType\s*:\s*(\S+)\s*$/)
    {
      $antSetup{'type'} = $1 ;
    }
    if (m/^\s*AntMethod\s*:\s*(\S+)\s*$/)
    {
      $antSetup{'method'} = $1+1 ;
    }
    if (m/^\s*AntSerial\s*:\s*(\S+)\s*$/)
    {
      $antSetup{'serial'} = $1 ;
    }
    if (m/^\s*AntHeight\s*:\s*(\S+)\s*$/)
    {
      $antSetup{'height'} = sprintf( "%.3f", $1 ) ;
    }
  }

  close( REFCONF ) ;

  # Return the parsed data hash to the caller via the reference
  # parameter.
  %{$refToHash} = %antSetup ;

} # end of getAntennaSetup() 



########################################################################
sub updateAntennaSetup
########################################################################
# Save the antenna values in the reference station configuration file.
# Several parameters are fed in:
#   $_[0] : serial number
#   $_[1] : antenna type
#   $_[2] : antenna height
#   $_[3] : height method
#   $_[4] : antenna name
# The current lines encoding these values in ref_stn.config are replaced
# with new ones containing the new data.
# Then the system is informed of the change through a call to 
# 'sysinfo -m'
########################################################################
{
  my ($serial, $type, $height, $method, $name ) = @_ ;
  # Open the existing file, and copy it line-by-line into a
  # copy in /tmp.  Leave out any lines containing the information
  # the user has just entered.
  open( NEWCONF, '>/tmp/ref_stn.config' ) ;

  # Copy the information from the existing config file,
  # leaving out the item that is to be deleted.
  if (open( OLDCONF, '/etc/sysconfig/ref_stn.config' ))
  {
    while (<OLDCONF>)
    {
      next if ( m/^\s*(AntSerial|AntType|AntName|AntMethod|AntHeight)\s*:/ ) ;
      print NEWCONF $_ ;
    }
  }
  close( OLDCONF ) ;

  printf NEWCONF "AntSerial:   %s\n", $serial ;
  printf NEWCONF "AntType:     %s\n", $type ;
  printf NEWCONF "AntName:     %s\n", $name ;
  printf NEWCONF "AntHeight:   %s\n", $height ;
  printf NEWCONF "AntMethod:   %s\n", ($method - 1) ;
  close( NEWCONF ) ;


  # Copy the new file on top of the old one.
  system( 
    '/usr/bin/sudo cp /tmp/ref_stn.config /etc/sysconfig/ref_stn.config'
        ) ;

  # Tell the sysinfo system to load in the revised data.
  system( 'sudo /usr/local/fireball/sysinfo -m' ) ;
 
} # end of updateAntennaSetup() 



########################################################################
sub getEthernetSetup
########################################################################
# Returns a hash containing the current Ethernet port state.
# Hash is returned via a hash-reference passed in via $_[0].
# Keys to the hash are teh various control names:
#   'ipmode'      : string either 'static' or 'dhcp'.
#   'ip'          : string containing the current IP address.
#   'netmask'     : string containing the current subnet mask.
#   'gateway'     : string containing the current gateway address.
#                   Will be '' if not set.
#   'nameServers' : Comma separated list of nameserver IPs, 
#                   '' if not set.
#   'dnsDomain'   : The DNS domain, '' if not set.
#   'dnsSearch'   : The DNS search path, '' if not set.
#   'mac'         : string describing the hardware MAC address
########################################################################
{
  my $hashRef = $_[0] ;

  # Open and read the current account configuration state.
  my %ethernetSetup ;

  # Read configuration from /etc/sysconf/interfaces
  # The line in question will look like one of these:
  #   iface eth0 inet dhcp 
  #   iface eth0 inet static
  # We want the fourth field.
  $ethernetSetup{'ipmode'} = 
       (split( ' ', 
               qx| grep '^ *iface  *eth0' /etc/sysconfig/interfaces| 
             ) 
       )[3] ;


  # Get the 12 character Hardware MAC address.  
  # Insert colons every two characters.
  my $macAddress = qx( /usr/local/fireball/getmac ) ;
  chomp $macAddress ;
  substr( $macAddress, 10, 0 ) = ':' ;
  substr( $macAddress,  8, 0 ) = ':' ;
  substr( $macAddress,  6, 0 ) = ':' ;
  substr( $macAddress,  4, 0 ) = ':' ;
  substr( $macAddress,  2, 0 ) = ':' ;
  $ethernetSetup{'mac'} = $macAddress ;


  # Determine the current Ethernet IP and Gateway.
  # If the network setup is okay, this will pop out of 'ifconfig' in the
  # form:
  #   inet addr:192.168.142.141 Bcast:192.168.142.255 Mask:255.255.255.0
  # The information is parsed from fields 2 and 4.
  # If the current network config has a problem, then ifconfig might not
  # be functional.  If we were in static mode, then we can read the 
  # ip and gateway from /etc/sysconfig/interfaces file. 
  # If all else fails, just default the IP to 0.0.0.0 and mask to
  # 255.255.255.0.
  my $ipAddress = '0.0.0.0' ;
  my $netMask = '255.255.255.0' ;
  my @results=split( ' ', 
                     qx( /sbin/ifconfig eth0 | grep 'inet addr:' ) 
                   ) ;
  if (@results)
  {
    $ipAddress  = (split( ':', $results[1] ))[1] ;
    $netMask  = (split( ':', $results[3] ))[1] ;
  }
  elsif ($ethernetSetup{'ipmode'} eq 'static')
  {
    $ipAddress = 
         (split( ' ', qx(grep address /etc/sysconfig/interfaces) ))[1] ;
    $netMask  =  
         (split( ' ', qx(grep netmask /etc/sysconfig/interfaces) ))[1] ;
  }

  $ethernetSetup{'ip'} = $ipAddress ;
  $ethernetSetup{'netmask'} = $netMask ;


  # Determine the Gateway setting.
  # If the current setup is okay, and uses a gateway, then this will
  # pop out of '/sbin/route' as the 'default'.  We want the value to
  # be numeric, so we use the -n option.  That makes "default" into
  # 0.0.0.0, so we search for that.  The line would look like:
  #   Destination  Gateway      Genmask   Flags Metric Ref    Use Iface
  #   0.0.0.0      155.63.20.1  0.0.0.0   UG    0      0        0 eth0
  # We grep for the starting 0.0.0.0 and then grab field number 2.
  # If that doesn't produce a default route then we try looking in
  # 'interfaces'.  There should be a specification if the mode is
  # static.  
  # If that fails then we just indicate an empty string.
  my $gateway = '' ;
  @results = grep( /^0.0.0.0/, qx| /sbin/route -n | ) ;
  if (@results)
  {
    $gateway = (split( / +/, $results[0] ))[1] ;
  }
  elsif ($ethernetSetup{'ipmode'} eq 'static')
  {
    $gateway = 
         (split( ' ', qx(grep gateway /etc/sysconfig/interfaces) ))[1] ;
  }

  $ethernetSetup{'gateway'} = $gateway ;


  # Determine the Name server setup.
  # This is controlled in the file /etc/resolv.conf.  
  #
  # The NameServers are defined on lines starting with 'nameserver '.
  # The values will always be numeric IPs.  There can be several.  We
  # allow up to three.  The grep below both detects lines starting with
  # 'nameserver ' AND removes those strings from the beginnings of the
  # lines.
  #
  # The domain is defined with a line starting with "domain ".  The
  # search path is defined with 

  # Read in the resolv.conf file, one line per element of an array.  If
  # the file doesn't exist, we'll get an empty array.
  @results = qx| cat /etc/resolv.conf | ;
  chomp( @results ) ;   # Drop all newline characters

  my @DnsServers = grep( s/^nameserver +//, @results ) ;

  my $domain = (grep( s/^domain +//, @results ))[0] ;

  my $searchPath = (grep( s/^search +//, @results ))[0];
  if ($searchPath)
  {
    $searchPath = join( ',', split( ' ', $searchPath ) ) ;
  }

  $ethernetSetup{'nameServers'} = join( ',', @DnsServers ) ;
  $ethernetSetup{'dnsDomain'} = $domain ;
  $ethernetSetup{'dnsSearch'} = $searchPath ;

  # Copy the created Hash into the location given by the 
  # parametric reference.
  %{$hashRef} = %ethernetSetup ;

} # end of getEthernetSetup()



########################################################################
sub updateEthernetSetup
########################################################################
# Takes a hash containing the desired Ethernet setup parameters, and
# updates the system configuration files to match.
# The parameter is a reference to a hash containing:
#   'ipmode'      : string either 'static' or 'dhcp'.
#   'ip'          : string containing the current IP address.
#   'netmask'     : string containing the current subnet mask.
#   'gateway'     : string containing the current gateway address.
#                   Will be '' if not set.
#   'nameServers' : Comma separated list of nameserver IPs, 
#                   '' if not set.
#   'dnsDomain'   : The DNS domain, '' if not set.
#   'dnsSearch'   : The DNS search path, '' if not set.
#   'mac'         : string describing the hardware MAC address
#
# This same hash is produced by getEthernetSetup().
#
########################################################################
{
  # Copy the referenced hash to a local version to make the 
  # code below easier to work with.
  my %ethernetSetup = %{$_[0]} ;

  my $ipmode = $ethernetSetup{ 'ipmode' } ;
  my $NewIP ;
  my $NewNetmask ;
  my $NewGateway ;
  my $network ;
  my $broadcast ;

  # If the new mode is static then we need to determine the
  # network base address and the broadcast address for later
  # inclusion in the interfaces file.
  if ($ipmode eq 'static')
  {
    $NewIP = $ethernetSetup{ 'ip' } ;

    $NewNetmask = $ethernetSetup{ 'netmask' } ;

    $NewGateway = $ethernetSetup{ 'gateway' } ;

    # Determine the subnet base address (network) based on the ip 
    # address and the netmask.  
    my @ipcomps = split( '\.', $NewIP ) ;
    my @maskcomps = split( '\.', $NewNetmask ) ;
    my @network ;
    $network[0] = ($ipcomps[0]+0) & $maskcomps[0] ;
    $network[1] = ($ipcomps[1]+0) & $maskcomps[1] ;
    $network[2] = ($ipcomps[2]+0) & $maskcomps[2] ;
    $network[3] = ($ipcomps[3]+0) & $maskcomps[3] ;
    $network = join( '.', @network) ;

    # Likewise determine the broadcast address for this subnet.
    my @broadcast ;
    $broadcast[0] = $network[0] | (255 ^ $maskcomps[0]) ;
    $broadcast[1] = $network[1] | (255 ^ $maskcomps[1]) ;
    $broadcast[2] = $network[2] | (255 ^ $maskcomps[2]) ;
    $broadcast[3] = $network[3] | (255 ^ $maskcomps[3]) ;
    $broadcast = join( '.', @broadcast) ;
  }

  # Re-generate the file /etc/sysconfig/interfaces
  # We read through the original file, transfering and filtering to a
  # new version in /tmp.  Comments are removed.  Lines not part of the
  # eth0 configuration are just copied.  
  # When we reach the iface-eth0 line, we regenerate that section.
  open( OLDIF, '/etc/sysconfig/interfaces' ) ;
  open( NEWIF, ">/tmp/interfaces" ) ;
  my $line ;
  MAINLOOP: while (<OLDIF>)
  {
    next MAINLOOP if m/^\s*#/ ;
    next MAINLOOP if m/^\s*$/ ;
    if ( m/^\s*iface\s+eth0/ )
    {
      if ($ipmode eq "static")
      {
        print NEWIF "iface eth0 inet static\n" ;
        print NEWIF "    address $NewIP\n" ;
        print NEWIF "    network $network\n" ;
        print NEWIF "    netmask $NewNetmask\n" ;
        print NEWIF "    broadcast $broadcast\n" ;
        print NEWIF "    gateway $NewGateway\n" ;
      }
      else
      {
        print NEWIF "iface eth0 inet dhcp\n" ;
      }
      SKIP: while (<OLDIF>)
      {
        next SKIP if (m/^\s*(address|network|netmask|broadcast|gateway|up|down)/) ;
        redo MAINLOOP ;   # Else Reprocess this line.
      }
    }
    else
    {
      print  NEWIF $_ ;
    }
  }
  close( NEWIF ) ;
  close( OLDIF ) ;

  if ($ipmode eq 'static')
  {
    open( RESOLV, ">/tmp/resolv.conf" ) ;

    print RESOLV "#\n" ;
    print RESOLV "# resolv.conf  This file is the resolver configuration file\n" ;
    print RESOLV "# See resolver(5).\n" ;
    print RESOLV "#\n" ;

    print RESOLV "# Updated : ", qx| /bin/date |, "\n" ;

    if ($ethernetSetup{ 'dnsDomain' })
    {
      print RESOLV "domain $ethernetSetup{'dnsDomain'}\n" ;
    }

    if ( my $path = $ethernetSetup{ 'dnsSearch' } )
    {
      # Replaces commas with spaces and reduce multiple spaces to
      # single spaces.
      $path =~ s/,/ /g ;
      $path =~ s/  */ /g ;
      
      print RESOLV "search ", $path, "\n" ;
    }

    for my $ns ( split( ',', $ethernetSetup{ 'nameServers' } ) ) 
    {
      print RESOLV "nameserver $ns\n" ;
    }
    close( RESOLV ) ;
    system( '/usr/bin/sudo cp /tmp/resolv.conf /etc/resolv.conf' ) ;
  }

  # Perform the actual network reconfiguration.
  # This is done as a background task, detached from the web
  # interface.  That allows the web page to finish transmission,
  # along with the warning message that the network change might
  # have disabled the Web interface.  The output from that
  # task is captured in /tmp/eth0reconfig.out, for debugging.  That
  # could be suppressed if desired.
  # The process for reconfiguring the network requires that we 
  # first disconnect any NFS mounts (or they would hang), and
  # reconnect them afterward.
  # The network systems are then stopped, with the old configuration
  # file in place, so it can shut down cleanly.  Then the new
  # config file is copied into place and networking is restarted.
  system( 
    '(
       /bin/sleep 5     # To let Web page finish updating.
       /bin/date
       /bin/echo "Installing new network configuration."
       /usr/bin/sudo /etc/init.d/networking stop 
       /usr/bin/sudo cp /tmp/interfaces /etc/sysconfig/interfaces 
       /usr/bin/sudo /etc/init.d/networking start 
       /bin/echo "Network reconfiguration is done."
       /bin/date
     ) > /tmp/eth0reconfig.out 2>&1 &' ) ;

} # end of updateEthernetSetup()


########################################################################
sub getSessionData
########################################################################
# Read all of the session data into a hash table, indexed by session
# name.  Each entry is a sub-hash containing the named parameters of the
# session.
# Return this hash to the caller via a reference-to-hash parameter.
########################################################################
{

  my $refHash = $_[0] ;

  my %sessData ;

  if ( open( SESSIONS, '/etc/sysconfig/sessions' ) )
  {
    while (<SESSIONS>)
    {
      next if (/^#/) ;
      next if (/^\s$/) ;
      my ($name, $enable, $sched, $time, $duration, $format, $naming )
            = split( ' ', $_ ) ;

      # If this is a "manual" session, then its enable status must
      # indicate 'off' after the session has finished.  This is 
      # necessary so that a session restart request will know that
      # it is truly a restart, and not just a re-affirmation of the
      # existing state.
      if ( $sched eq 'manual'
        && $enable == 1
         )
      {
        # Get the time of day in GPS seconds.  
        # 315964800 converts from Unix time to gps time (01Jan1970 to
        # 06Jan1980).  
        # Include a 13 second offset for the UTC to GPS leapseconds.
        # Note that this offset does NOT have to be exact and could
        # tolerate numerous leap sections before we'd need to update it.
        # Ideally, the current value would be read from the system
        # somehow.
        my $gpsTime = time() - 315964800 + 13 ;

        if ($gpsTime > $time+60*$duration)
        {
          # We are past the time for this manual session.  Force
          # its state to disabled.
          $enable = 0 ;
        }
      }

      $sessData{$name}{'enable'} = $enable ;
      $sessData{$name}{'sched'} = $sched ;
      $sessData{$name}{'time'} = $time ;
      $sessData{$name}{'duration'} = $duration ;

      my ($ftype, $mrate, $prate, $option1, $option2 ) 
                 = split( ':', $format ) ;
      $sessData{$name}{'ftype'} = $ftype ;
      $sessData{$name}{'mrate'} = $mrate ;
      $sessData{$name}{'prate'} = $prate ;
      $sessData{$name}{'option1'} = $option1 ;
      $sessData{$name}{'option2'} = $option2 ;

      my ($sessid, $perday, $persessid, undef ) = split( ':', $naming );
      $sessData{$name}{'sessid'} = $sessid ;
      $sessData{$name}{'perday'} = $perday ;
      $sessData{$name}{'persessid'} = $persessid ;

    }
    close( SESSIONS ) ;
  }

  # Return the session data table to the caller via the reference.
  %{$refHash} = %sessData ;

} # end of getSessionData()



########################################################################
sub updateSessionData
########################################################################
# Recreate the the session data table based on a hash table, as created
# by getSessionData.  This routine reverses that process by formatting
# the hash back into the config file.
# Parameter $_[0] is a reference to a hash that is passed into this
# routine from the caller.
########################################################################
{
  my %sessData = %{$_[0]} ;

  open( SESSIONS, '>/tmp/sessions' ) ;
  print SESSIONS "#Name  Enb Type    Date/Time "
                ."Duration DataType        FileTemplate\n" ;

  for my $name (sort keys %sessData)
  {
    my $format = sprintf( '%s:%s:%s:%s:%s',
                          $sessData{$name}{'ftype'},
                          $sessData{$name}{'mrate'},
                          $sessData{$name}{'prate'},
                          $sessData{$name}{'option1'},
                          $sessData{$name}{'option2'},
                        ) ;

    printf( SESSIONS "%-7s %1d  %-6s %10d %4d     %-15s %s:%s:%s:0\n", 
            $name, 
            $sessData{$name}{'enable'}, 
            $sessData{$name}{'sched'}, 
            $sessData{$name}{'time'}, 
            $sessData{$name}{'duration'}, 

            $format,

            $sessData{$name}{'sessid'},
            $sessData{$name}{'perday'},
            $sessData{$name}{'persessid'},
          ) ;
  }

  system( 'sudo cp /tmp/sessions /etc/sysconfig/sessions' ) ;

  # Send signal 10 (USR1 = User defined signal 1) to the
  # scheduler to inform it that the schedule configuration has
  # been changed.
  if ( -f '/var/run/logScheduler.pid' )
  {
    my $spid = qx( cat /var/run/logScheduler.pid ) ;
    #print "Signalling $spid <br>\n" ;
    system( "/usr/bin/sudo /bin/kill -SIGUSR1 $spid" ) ;
  }

  close( SESSIONS ) ;

} # end of updateSessionData()



########################################################################
sub EnaDisSession
########################################################################
# Used by several routines to enable or disable a session.
# $_[0] = the session name.
# $_[1] = true:enable false:disable
# Returns true if successful.
# Returns false if failed, which should only happen if the session
# doesn't exist.
# Doesn't print anything to stdout.  The command handler that calls
# this subroutine must handle that.
{
  my $name = $_[0] ;
  my $ena = ($_[1] ? 1 : 0) ;

  # Read in the data from the sessions configuration file.
  my %sessData ;
  getSessionData( \%sessData ) ;

  return 0 if (! exists( $sessData{$name} ) ) ;

  $sessData{$name}{'enable'} = $ena ;
  
  # If this is a manual start session and the session is being
  # enabled, we want to record the current time as the start time
  # of the session.  A little margin is added so that the session
  # can start cleanly and get its full duration.
  if ( $sessData{$name}{'sched'} eq 'manual' 
    && $ena eq '1'
     )
  {
    # Get the time of day in GPS seconds.  Includes a 13 second
    # offset for the UTC to GPS leapseconds.  It would be nice to
    # have the 13 second value be obtained from a system utility,
    # but that's not available yet.  But even if the value changes
    # by a few seconds, it won't matter.  All that we want is for
    # the time to start the session to be in the future a little
    # bit - about 10 seconds.  Won't matter if that's 9 or 8 or 7.
    my $gpsTime = time() - 315964800 + 13 ;

    $sessData{$name}{'time'} = $gpsTime + 10 ;
  }

  updateSessionData( \%sessData ) ;

  return 1 ; # Indicating we found the session and did the request

} # end of EnaDisSession()



########################################################################
sub getPortSetup
########################################################################
# Returns a hash (via a reference parameter) containing a description
# of the complete I/O port setup.
# 
# Need a bunch more documentation here.
########################################################################
{
  my $refHash = $_[0] ;

  my %allPortData ;

  # For each of the serial ports, Set defaults and go through the 
  # PPP config files to set the defaults.  These can be overridden
  # when we later go through gpssd.config and find that the port 
  # isn't set to PPP mode.
  for my $port ( 1..4 )
  {
    my $name = "SerialPort$port" ;

    # Create some defaults for the serial ports, in case the
    # file has nothing in it for them.
    $allPortData{$name}{'protocol'} = 'serial' ;
    $allPortData{$name}{'service'} = 'ppp' ;
    $allPortData{$name}{'baud'} = 115200 ;
    $allPortData{$name}{'flow'} = 'None' ;
    $allPortData{$name}{'parity'} = 'None' ;
    $allPortData{$name}{'pppbaud'} = 115200 ;
    $allPortData{$name}{'pppflow'} = 'None' ;

    my $pppFile ;
    $pppFile = 'console'  if ($port == 1) ;
    $pppFile = 'tts.1'    if ($port == 2) ;
    $pppFile = 'tts_max0' if ($port == 3) ;
    $pppFile = 'tts_max1' if ($port == 4) ;

    # Defaults in case the options file doesn't exist.
    my $localip = '0.0.0.0' ;
    my $remoteip = '0.0.0.0' ;
    my $flow = 'None' ;

    if (open(PPP, "/etc/sysconfig/ppp/options.$pppFile" ))
    {
      while (<PPP>)
      {
        chomp ;
        if (m/:/)
        {
          ( $localip, $remoteip ) = split ':' ;
        }
        $flow = 'None' if (m/nocrtscts/ ) ;
        $flow = 'Hardware' if (m/^\s*crtscts/ ) ;
      }
      close( PPP ) ;
    }
    $allPortData{$name}{'localip'} = $localip ;
    $allPortData{$name}{'remoteip'} = $remoteip ;
    $allPortData{$name}{'pppflow'} = $flow ;
  }


  ######################################################################
  # Determine any PPP settings present in mgetty.config.
  ######################################################################
  if (open(MGETTY, '/etc/sysconfig/mgetty+sendfax/mgetty.config'))
  {
    my $line ;
    while ($line = <MGETTY>)
    {
      $_ = $line ;
      next if (m/^#/) ;
      next if (m/^\s*$/) ;
      if (m/^\s*port\s/ )
      {
        # This is the section describing an Mgetty device.
        #print ">>>> " ; print ;
        my (undef, $device) = split ;
        #print "Device = $device\n" ;
        my $externalPortNum ;
        $externalPortNum = 1 if ($device eq 'console') ;
        $externalPortNum = 3 if ($device eq 'tts_max0') ;
        $externalPortNum = 4 if ($device eq 'tts_max1') ;
        $externalPortNum = 2 if ($device eq 'tts/1') ;
        my $internalPortNum = $externalPortNum - 1 ;
        my $portName = "SerialPort$externalPortNum" ;

        # Default settings in case they aren't specfied.
        my $baud   = 115200 ;
        my $direct = 'y' ;
        my $reset = '' ;
        my ( $init1, $init2, $init3 ) = ('','','') ;

        # Parse this port section
        while ($line = <MGETTY>)
        {
          $_ = $line ;
          last if (m/^\s*$/) ;
          if (m/^\s*port\s/ )
          {
            $line = $_ ;
            last ;
          }
          #print ;
          if (m/speed/)
          {
            (undef, $baud ) = split ;
          }
          if (m/direct/)
          {
            (undef, $direct ) = split ;
          }
          if (m/force-init/)
          {
            (undef, undef, $reset, undef ) = split ;
          }
          if (m/init-chat/)
          {
            (undef, undef, $init1, undef, $init2, undef, $init3) 
                        = split ;
          }
        }

        # Note that baud may be overridden by gpssd.config, if the
        # port isn't servicing PPP.
        $allPortData{$portName}{'pppbaud'} = $baud ;

        $allPortData{$portName}{'connection'} = $direct eq 'y' 
                                             ? 'Direct' 
                                             : 'Modem' ;
        $allPortData{$portName}{'reset'} = $reset ;
        $allPortData{$portName}{'init1'} = $init1 ;
        $allPortData{$portName}{'init2'} = $init2 ;
        $allPortData{$portName}{'init3'} = $init3 ;

        redo if ($line =~ m/^\s*port\s/ ) ;
      }
    }
    close( MGETTY ) ;
  }



  # Read the settings from gpssd.config, which defines
  # almost everything for the non-PPP ports.
  if ( open( GPSSD, '/etc/sysconfig/gpssd.config' ) )
  {
    while (<GPSSD>)
    {
      next if (/^#/) ;
      next if (/^\s$/) ;
      my ( $port, $protocol, 
           $field3, $field4, $field5, $field6, $field7, $field8, 
           $field9, undef 
         ) = split( ' ', $_ ) ;

      my $service = $field5 ;

      # The metdata port is a hidden service we don't want to display.
      # Probably will go away in future releases.
      next if ($service eq 'metdata') ;


      my $name ;
      if ( $protocol eq 'serial')
      {
        ++$port ; # Ports in the file are 0..3, to the user 1..4
        $name = "SerialPort$port" ;
        $allPortData{$name}{'protocol'} = 'serial' ;

        my ($baud, $parity, $flow ) = split( ':', $field3 ) ;

        $allPortData{$name}{'baud'} = $baud ;

        $parity = 'None' if $parity eq 'N' ;
        $parity = 'Odd'  if $parity eq 'O' ;
        $parity = 'Even' if $parity eq 'E' ;
        $allPortData{$name}{'parity'} = $parity ;

        $flow = 'None' if $flow eq 'N' ;
        $flow = 'Hardware' if $flow eq 'H' ;
        $flow = 'XonXoff' if $flow eq 'S' ;
        $allPortData{$name}{'flow'} = $flow ;

        if ($service eq 'ppp')
        {
          # If a ppp configuration is changed in one of those other
          # files, then the gpssd system needs to be advised of it.
          # To do this, we will toggle the field6 values between
          # 1 and 10.  Thus we have to record the value as found.
          $allPortData{$name}{'ppptoggle'} = $field6 ;
        }
      }
      elsif ($protocol eq 'tcp')
      {
        $name = "TcpPort$port" ;
        $allPortData{$name}{'protocol'} = 'tcp' ;
      }
      elsif ($protocol eq 'udp')
      {
        $name = "UdpPort$port" ;
        $allPortData{$name}{'protocol'} = 'udp' ;
        $allPortData{$name}{'udptimeout'} = $field4 ;
      }


      $service = 'Trimcom' if ($service eq 'trimcom') ;
      $service = 'Met' if ($service eq 'met') ;

      if ($service eq 'rt17')
      {
        $service = 'RT17' ;
        my $interval = $field6 / 10 ;
        if ($interval < 1.0) 
        {
          $interval = sprintf( "%.1f", $interval) 
        }
        else
        {
          $interval = sprintf( "%.0f", $interval) 
        }
        $allPortData{$name}{'interval'} = $interval ;

        if ($field7 & 8)
        {
          # Pos Every Cycle is set.  
          # => want measurements and positions every epoch.
          $allPortData{$name}{'positions'} = 1 ;
          $allPortData{$name}{'measurements'} = 1 ;
        }
        elsif ($field7 & 16)
        {
          # Pos Only is set.
          # => want positions every cycle, but no measurements.
          $allPortData{$name}{'positions'} = 1 ;
          $allPortData{$name}{'measurements'} = 0 ;
        }
        else
        {
          # No special position options.
          # => want measurements every cycle, but no positions.
          $allPortData{$name}{'positions'} = 0 ;
          $allPortData{$name}{'measurements'} = 1 ;
        }

        $allPortData{$name}{'concise'} = ($field7 & 1)  ;
        $allPortData{$name}{'iodeslips'} = ($field7 & 2) >> 1  ;
        $allPortData{$name}{'gpsdata'} = ($field7 & 4) >> 2  ;
        $allPortData{$name}{'rawgps'} = ($field7 & 32) >> 5  ;
        $allPortData{$name}{'rawwaas'} = ($field7 & 64) >> 6  ;
        $allPortData{$name}{'smoothcode'} = ($field7 & 128) >> 7  ;
        $allPortData{$name}{'smoothcarrier'} = ($field7 & 256) >> 8  ;
      }
      if ($service eq 'binex')
      {
        $service = 'Binex' ;
        my $interval = $field6 / 10 ;
        if ($interval < 1.0) 
        {
          $interval = sprintf( "%.1f", $interval) 
        }
        else
        {
          $interval = sprintf( "%.0f", $interval) 
        }
        $allPortData{$name}{'interval'} = $interval ;
        $allPortData{$name}{'smoothing'} = ($field7 & 1) ;
      }
      elsif ($service eq 'cmr')
      {
        $service = 'CMR' ;
        $allPortData{$name}{'style'} = ($field7 == 1) 
                                       ? 'Standard' 
                                       : 'CmrPlus' ;
      }
      elsif ($service eq 'rtcm')
      {
        $service = 'Rtcm' ;
        my $version ;
        $version = '2.1' if ($field8 == 0) ;
        $version = '2.2' if ($field8 == 1) ;
        $version = '2.3' if ($field8 == 2) ;
        $version = '3.0' if ($field8 == 3) ;
        $allPortData{$name}{'version'} = $version ;

        my $style ;
        $style = 'DGPS' if ($field7 == 0) ;
        $style = 'RTK' if ($field7 == 1) ;
        $style = 'DGPS_RTK' if ($field7 == 2) ;
        if ($field7 == 3)
        {
          # custom style based on bit fields in $field9
          $style = 'Custom:' ;
          $style .= '1,' if ($field9 & 0x001) ;
          $style .= '2,' if ($field9 & 0x002) ;
          $style .= '3,' if ($field9 & 0x004) ;
          $style .= '9,' if ($field9 & 0x008) ;
          $style .= '14,' if ($field9 & 0x010) ;
          $style .= '18+19,' if ($field9 & 0x020) ;
          $style .= '22,' if ($field9 & 0x040) ;
          $style .= '23+24,' if ($field9 & 0x180) ;
          $style .= '59.3,' if ($field9 & 0x200) ;
          $style .= '59.13,' if ($field9 & 0x400) ;
          $style =~ s/,$// ; # drop the final comma
        }
        $allPortData{$name}{'style'} = $style ;
      }
      elsif ($service eq 'service')
      {
        $service = 'SvStatus' if ($field6 == 0) ;
        $service = 'Ephemerides' if ($field6 == 1) ;
        $service = 'Almanacs' if ($field6 == 2) ;
        $service = 'internal3' if ($field6 == 3) ; 
        $service = 'internal4' if ($field6 == 4) ;
      }
      $allPortData{$name}{'service'} = $service ;

      $allPortData{$name}{'field3'} = $field3 ;
      $allPortData{$name}{'field4'} = $field4 ;
      $allPortData{$name}{'field5'} = $field5 ;
      $allPortData{$name}{'field6'} = $field6 ;
      $allPortData{$name}{'field7'} = $field7 ;
      $allPortData{$name}{'field8'} = $field8 ;
      $allPortData{$name}{'field9'} = $field9 ;

    }
  }
  close( GPSSD ) ;


  ######################################################################
  # Read the contents of the Met/Tilt configuration file.  
  # This is an array of up to six controls, with a time-specification
  # and a command string for each.  We'll encode these into 
  # $portData{'MetControls'}, which is NOT a normal port, and must be
  # filtered out of any 'keys %portData' constructs.
  ######################################################################

  # Create the array of commands, and default them all to Off.
  for my $n (1..6)
  {
    $allPortData{'MetControls'}{$n} = "off" ;
  }

  if (open( METCONF, '/etc/sysconfig/met.conf' ))
  {
    my $i = 1 ;
    while (<METCONF>)
    {
      chomp ;
      my ($time, $cmd) = split ;
      $cmd = '' if (!$cmd ) ;
      $time = 'once' if ($time == 0) ;
      $allPortData{'MetControls'}{$i++} = "$time:$cmd" ;
    }
    close( METCONF ) ;
  }


  # Return the configuration hash to the caller via the reference.
  %{$refHash} = %allPortData ;

} # end of getPortSetup



########################################################################
sub saveGpssdConfig
########################################################################
# Re-encodes the port setup information from the %portData hash
# structure back into the gpssd.config configuration file.
# The %portData structure is passed in via $_[0], a reference parameter.
#
# Need a bunch more documentation here.
########################################################################
{
  # Copy the referenced hash to a local version to make the 
  # code below easier to work with.
  my %allPortData = %{$_[0]} ;

  open( GPSSD, '>/tmp/gpssd.config' ) ;

  # If we have a Met port on one of the serial ports, then we'll
  # need to generate an internal-use-only metdata line in the gpssd
  # file.  When this is detected, this flag is set.
  my $metFound = 0 ;

  for my $name (sort keys %allPortData)
  {
    # Skip the MetControls port since it isn't a real port.
    next if ( $name eq 'MetControls' ) ;

    # For convenience, make a single hash of this port's data
    my %portData = %{$allPortData{$name}} ;

    # Extract the port number
    my $portNumber = $name ;
    $portNumber =~ s/.*Port// ;

    # Extract the protocol from the name.
    # This could be obtained from $portData{'protocol'}, assuming that
    # is always set.  That value should always match the port's
    # name, but it is more robust to figure it out.  Several times
    # during development the protocol was forgotten when new ports
    # were created, leading to a corrupt gpssd.config.
    my $protocol = lc($name) ;
    $protocol =~ s/^(tcp|udp|serial).*$/$1/ ;

    # Decrement the Serial port numbers.  We present the user with ports
    # 1 to 4, but the file wants 0 to 3
    --$portNumber if ($protocol eq 'serial') ;

    my $baud = $portData{'baud'} || 0 ;
    if ($baud != 0)
    {
      my $flow = $portData{'flow'} ;
      $flow = 'N' if ($flow =~ /none/i) ;
      $flow = 'H' if ($flow =~ /hardware/i) ;
      $flow = 'S' if ($flow =~ /xonxoff/i) ;

      my $parity = $portData{'parity'} ;
      $parity = 'N' if ($parity =~ /none/i) ;
      $parity = 'E' if ($parity =~ /even/i) ;
      $parity = 'O' if ($parity =~ /odd/i) ;

      $baud .= ":$parity:$flow" ;
    }

    # UdpTimeout will end up in field 4.
    # If set in the hash, use that.  Otherwise don't-care it to zero.
    # Note that this appears to be used on some non-UDP ports, but 
    # I don't know what for exactly.
    my $udptimeout = $portData{'udptimeout'} || 0 ;

    # Determine the service-specific values in Fields 5 thru 9.
    # The service is always encoded in lower-case.
    #
    # Some services are re-encoded into 'service' and a sub-service
    # value in $field6.  These will be commented in the output showing
    # the true service value.
    my $service = lc( $portData{'service'} ) ;

    my $field6 = 0 ;
    my $field7 = '' ;
    my $field8 = '' ;
    my $field9 = '' ;

    if ($service eq 'svstatus' )
    {
      print GPSSD "# service 0 = SV status\n" ;
      $service = 'service' ;
      $field6 = 0 ; # sub-service type
    }
    elsif ($service eq 'ephemerides' )
    {
      print GPSSD "# service 1 = Ephemerides\n" ;
      $service = 'service' ;
      $field6 = 1 ; # sub-service type
    }
    elsif ($service eq 'almanacs' )
    {
      print GPSSD "# service 2 = Almanacs\n" ;
      $service = 'service' ;
      $field6 = 2 ; # sub-service type
    }
    elsif ($service eq 'internal3' )
    {
      print GPSSD "# service 3 = IOport Status\n" ;
      $service = 'service' ;
      $field6 = 3 ; # sub-service type
      $field7 = 3 ; # unknown purpose
      $udptimeout = 45 ; #unknown purpose
    } 
    elsif ($service eq 'internal4' )
    {
      print GPSSD "# service 4 = Mfg Test\n" ;
      $service = 'service' ;
      $field6 = 4 ; # sub-service type
      $field7 = 4 ; # unknown purpose
    }
    elsif ($service eq 'rt17' )
    {
      $field6 = $portData{'interval'} * 10 ;

      $field7 = 0 ;
      $field7 |= 1   if ($portData{'concise'}) ;
      $field7 |= 2   if ($portData{'iodeslips'}) ;
      $field7 |= 4   if ($portData{'gpsdata'}) ;
      $field7 |= 32  if ($portData{'rawgps'}) ;
      $field7 |= 64  if ($portData{'rawwaas'}) ;
      $field7 |= 128 if ($portData{'smoothcode'}) ;
      $field7 |= 256 if ($portData{'smoothcarrier'}) ;

      # "Include positions with measurements" bit
      $field7 |= 8   if ( $portData{'positions'}
                       && $portData{'measurements'}
                        ) ;
      # "Positions only" bit                        
      $field7 |= 16  if ( $portData{'positions'}
                       && ! $portData{'measurements'}
                        ) ;
    }
    elsif ($service eq 'binex' )
    {
      $field6 = $portData{'interval'} * 10 ;
      $field7 = $portData{'smoothing'} ? 1 : 0 ;
    }
    elsif ($service eq 'met' )
    {
      $field6  = '10' ;
      $field7  = '0' ;
      $field8  = '0' ;
      $metFound = 1 ;
    }
    elsif ($service eq 'trimcom' )
    {
      $field6  = '10' ;
      $field7  = '0' ;
    }
    elsif ($service eq 'cmr' )
    {
      $field6 = '10' ;
      my $style = lc($portData{'style'}) ;
      $field7  = $style eq 'standard' ? 1 : 2 ;
    }
    elsif ($service eq 'rtcm' )
    {
      # Field 6 is frequency.  Always set to 10 (1 Hz)
      $field6 = 10 ;

      my $version = $portData{'version'} ;
      $field8 = 0 ; # Default, as if ($version eq '2.1') 
      $field8 = 1 if ($version eq '2.2') ;
      $field8 = 2 if ($version eq '2.3') ;
      $field8 = 3 if ($version eq '3.0') ;

      # Field 7 is the "style"
      my $style = lc($portData{'style'}) ;
      $field7 = 0 ; # Default, should be if ($style eq 'DGPS') 
      $field9 = 0 ; # default this in case unset below.

      $field7 = 1 if ($style eq 'rtk') ;
      $field7 = 2 if ($style eq 'dgps_rtk') ;
      if ($style =~ /custom/)
      {
        $field7 = 3 ;
      
        # Custom styles are indicated with a format of:
        #     Custom:1,3,9,...
        # Drop the Custom: and split the rest into a list
        # on commas.  Then add bits to $field9 to match those.
        $style =~ s/custom://i ;
        #print "Style=$style<br>\n" ;
        for my $bit (split( ',', $style ) )
        {
          $field9 |= 1    if ($bit eq '1') ;
          $field9 |= 2    if ($bit eq '2') ;
          $field9 |= 4    if ($bit eq '3') ;
          $field9 |= 8    if ($bit eq '9') ;
          $field9 |= 16   if ($bit eq '14') ;
          $field9 |= 32   if ($bit eq '18+19') ;
          $field9 |= 64   if ($bit eq '22') ;
          $field9 |= 384  if ($bit eq '23+24') ;
          $field9 |= 512  if ($bit eq '59.3') ;
          $field9 |= 1024 if ($bit eq '59.13') ;
        }
      }
    }
    elsif ($service eq 'ppp' )
    {
      # Field 6 is the ppp toggle byte.  This is used to trigger
      # restart of mgetty/ppp on ports that have changed their
      # PPP configurations. Since that data isn't in gpssd.config,
      # but the ppp-restart is done by gpssd, we have to inform it
      # of a change.  So we toggle field 6 between 1 and 10, just
      # to make something appear to change.  If not set, we default
      # it to '1'.
      $field6 = $portData{'ppptoggle'} || 1 ;
      $field7 = 0 ;
    }

    # Finally we can emit the full composite line for this port...
    printf GPSSD "%-4d %-6s %10s %2d %-7s %s %s %s %s\n", 
           $portNumber, $protocol, $baud, $udptimeout, $service, 
           $field6, $field7, $field8, $field9 ;
  }

  # If we found a met port above, then we need to set up the
  # internal-use-only metdata port.  This will eventually go away.
  if ($metFound) 
  {
    print GPSSD "8099 tcp             0  0 metdata 1 0\n" ;
  }
  
  close( GPSSD ) ;

  # For debugging: Emit the config file
  #print qx( cat /tmp/gpssd.config ) ;

  # Copy the new version on top of the old one.
  # Copy preserves all permissions of the original.
  # Sudo is needed because the Apache user netrx can't
  # write in the sysconfig directories.  
  system( '/usr/bin/sudo /bin/cp '
         .'/tmp/gpssd.config /etc/sysconfig/gpssd.config' ) ;

  # Restart the existing gpssd daemon to make the changes take
  # effect.  This is done by signaling the process to re-read its
  # config file, which will detect any changes and only affect the
  # modified streams and ports.  
  if ( -e '/var/run/gpssd.pid' )
  {
    # Get the process idea from the file.
    my $pid = qx( cat /var/run/gpssd.pid ) ;
    #print "Signalling gpssd (PID=$pid)<br>" ;
    system( "/usr/bin/sudo kill -SIGHUP $pid" ) ;
  }

} # end of saveGpssdConfig()



########################################################################
sub saveMetConfig
########################################################################
# Re-encodes the Met service configuration data from the %allPortData 
# hash structure back into the /etc/sysconfig/met.conf file.
#
# The %allPortData structure is passed in via $_[0], a reference
# parameter.
########################################################################
{
  # Copy the referenced hash to a local version to make the 
  # code below easier to work with.
  my %allPortData = %{$_[0]} ;

  open( METCONF, '>/tmp/met.conf' ) ;

  for my $n (1..6)
  {
    my $cmd = $allPortData{'MetControls'}{$n} ;
    next if ($cmd eq 'off') ;

    my ($time, $string) = split( ':', $cmd ) ;
    $time = 0 if ( $time =~ /^once$/i ) ;

    print METCONF "$time $string\n" ;
  }

  close( METCONF ) ;

  # Copy the new version on top of the old one.
  # Copy preserves all permissions of the original.
  # Sudo is needed because the Apache user netrx can't
  # write in the sysconfig directories.  
  system( '/usr/bin/sudo /bin/cp '
         .'/tmp/met.conf '
         .'/etc/sysconfig/met.conf' ) ;

} # end of saveMetConfig()



########################################################################
sub savePppConfig
########################################################################
# Re-encodes the PPP configuration data %portData hash
# structure back into the mgetty/ppp configuration files.
# The %portData structure is passed in via $_[0], a reference parameter.
#
# Need a bunch more documentation here.
########################################################################
{
  # Copy the referenced hash to a local version to make the 
  # code below easier to work with.
  my %allPortData = %{$_[0]} ;

  # Update the mgetty configuration file
  open( MGETTY, '/etc/sysconfig/mgetty+sendfax/mgetty.config') ;
  open( NEWMGETTY, '>/tmp/mgetty.config' ) ;

  # Read through the file.  Default behavior is to copy each line
  # to the new file.  When we hit the "port" specification section
  # at the end of the file, we stop transfering and start regeneration.
  # Note that it is a requirement on the file that the port section is
  # at the end, so we can just toss it all from the first port section
  # on.
  while (<MGETTY>)
  {
    last if (/^\s*port\s/) ;
    print NEWMGETTY ;
  }
  close MGETTY ;

  ######################################################################
  # Regnerate the port configurations with a section for each of our
  # serial ports.  A section looks like this:
  #
  #     # SerialPort1
  #     port console
  #       direct n
  #       speed 115200
  #       toggle-dtr n
  #       force-init "" RST
  #       init-chat "" IN1 OK IN2 OK IN3 OK
  #       debug 0
  #
  ######################################################################
  for my $p (1..4)
  {
    my $port = "SerialPort$p" ;
    print NEWMGETTY "# $port\n" ;

    # Map our port numbers to device names
    my $device ;
    $device='console' if ($p == 1) ;
    $device='tts/1' if ($p == 2) ;
    $device='tts_max0' if ($p == 3) ;
    $device='tts_max1' if ($p == 4) ;
    print NEWMGETTY "port $device\n" ;

    # a line stating 'direct y' or 'direct n'
    print NEWMGETTY "  direct ", 
                       ($allPortData{$port}{'connection'} =~ /Direct/i)
                         ? 'y' 
                         : 'n', 
                       "\n" ;

    # The ppp baud rate
    print NEWMGETTY "  speed ", 
                       $allPortData{$port}{'pppbaud'},
                       "\n" ;

    # A fixed setting for one of the parameters.
    print NEWMGETTY "  toggle-dtr n\n" ;


    # If user specifies a resetstring, emit it in the form:
    #     force-init "" string
    my $resetString = $allPortData{$port}{'reset'} ;
    if ( length($resetString) )
    {
      print NEWMGETTY '  force-init "" ', $resetString, "\n" ;
    }

    # If user specifies one or more initStrings, emit them in the form:
    #     init-chat "" string OK string OK string OK
    my $initString1 = $allPortData{$port}{'init1'} ;
    $initString1 = '' if (!defined $initString1) ;

    my $initString2 = $allPortData{$port}{'init2'} ;
    $initString2 = '' if (!defined $initString2) ;

    my $initString3 = $allPortData{$port}{'init3'} ;
    $initString3 = '' if (!defined $initString3) ;

    if ( length( $initString1 ) 
      or length( $initString2 ) 
      or length( $initString3 )
       )
    {
      print NEWMGETTY '  init-chat ""' ;
      print NEWMGETTY " $initString1 OK" if ( length( $initString1 ) ) ;
      print NEWMGETTY " $initString2 OK" if ( length( $initString2 ) ) ;
      print NEWMGETTY " $initString3 OK" if ( length( $initString3 ) ) ;
      print NEWMGETTY "\n" ;
    }

    # A fixed setting for the debug level.
    print NEWMGETTY "  debug 0\n";

    print NEWMGETTY "\n" ;
  }

  close NEWMGETTY ;

  # Copy the new mgetty config file on top of the old one.
  # Copy preserves all permissions of the original.
  # Sudo is needed because the Apache user netrx can't
  # write in the sysconfig directories.  
  system( '/usr/bin/sudo /bin/cp '
         .'/tmp/mgetty.config '
         .'/etc/sysconfig/mgetty+sendfax/mgetty.config' ) ;

  
  ######################################################################
  # Regenerate the ppp configuration files, one for each port.
  ######################################################################
  for my $p (1..4)
  {
    my $port = "SerialPort$p" ;

    # Map our port numbers to ppp file names
    my $pppFile ;
    $pppFile='console' if ($p == 1) ;
    $pppFile='tts.1' if ($p == 2) ;
    $pppFile='tts_max0' if ($p == 3) ;
    $pppFile='tts_max1' if ($p == 4) ;
    $pppFile="options.$pppFile" ;

    open( NEWPPPFILE, ">/tmp/$pppFile" ) ;
    if (open( OLDPPPFILE, "/etc/sysconfig/ppp/$pppFile" ) )
    {
      while (<OLDPPPFILE>)
      {
        next if ( m/:/ ) ;     # Skip local:remote line.
        next if (m/local/) ;   # Skip "local" if it exists
        next if (m/crtscts/) ; # Skip flow control if it exists
        next if (m/^\s$/) ;    # Skip blank lines
        print NEWPPPFILE $_ ;
      }
      close OLDPPPFILE ;
    }
    # The new ppp options file now has been stripped of all the lines
    # under GUI control, leaving anything else behind.  Next we add
    # in the lines under GUI control, as needed.

    my $flow= $allPortData{$port}{'pppflow'} ;
    $flow = ($flow =~ /hardware/i)
            ? 'crtscts' 
            : 'nocrtscts' ;

    print NEWPPPFILE $allPortData{$port}{'localip'}, 
                     ':', 
                     $allPortData{$port}{'remoteip'}, 
                     "\n",
                     $flow,
                     "\n" ;

    print NEWPPPFILE "local\n" 
        if ( $allPortData{$port}{'connection'} =~ /Direct/i ) ;

    close PPPFILE ;

    # Copy the new mgetty config file on top of the old one.
    # Copy preserves all permissions of the original.
    # Sudo is needed because the Apache user netrx can't
    # write in the sysconfig directories.  
    system( '/usr/bin/sudo /bin/cp '
           ."/tmp/$pppFile "
           ."/etc/sysconfig/ppp/$pppFile" ) ;
  }

} # end of saveGpssdConfig()


########################################################################
sub sendBinaryFile
########################################################################
# Sends a binary file as the CGI response.
# The path name of the file is given in $_[0].
# The file is sent as type "application/octet-stream, which triggers
# 'download and save' on most browsers.
# It is assumed that the file's existence has already been tested.
# If the file can't be read, we'll probably return a zero length file.
########################################################################
{
  my $pathname = $_[0] ;

  # Figure out the suggested filename for the remote computer to use,
  # by dropping any /leading/path/information.
  my $fname = $pathname ;
  $fname =~ s/^.*\/// ;            

  my $length = (-s $pathname) ;    # Length of file in bytes.
  
  # This is the tricky part.  We aren't sending back a web page.
  # Instead we want to send a stream of bytes and have the browser 
  # save those bytes in a file.  This is triggered by some special
  # items in the HTTP header sent before the content.  In particular,
  # we want this file to be of type application/octet-stream, to trigger
  # the download, and specify that the data is an "attachment" destined
  # for a suggested name.  
  # We also specify the length of the file, which is used by some
  # browsers to provide a download-progress indicator.

  print header( '-type' => 'application/octet-stream',
                '-attachment' => $fname,
                '-Content-length' => $length,
                '-expires' => '+1s'
              ) ;

  # Now we just send the raw bytes of the file.  We use perl's
  # block read function, to grab 4kbytes at a time and just send it
  open THEFILE,  $pathname  || return ;
  my $buf ;
  while ( read( THEFILE, $buf, 4096 ) )
  {
    print $buf ;
  }
  
  # I'm not sure how the device on the other end knows when the file is
  # done.  Perhaps the apache server notices when this script exits and
  # closes the channel.  I'm pretty sure it has nothing to do with the
  # length value we sent above.  Things still work just fine when that
  # is removed.  Another little mystery.

} # end of sendBinaryFile() */



########################################################################
sub getRefStationData
########################################################################
# Returns a hash containing the reference station information.
# Elements are:
#   lat lon height 
#   x y z
#   rtcmid
#   cmrid
#   rtcmname
#   rtcmdesc
# The hash is returned via a reference parameter in $_[0] ;
########################################################################
{
  my %refStationData ;

  open( SYSINFO, 
        '/usr/bin/sudo /usr/local/fireball/sysinfo -p ref_stn |' 
      ) ;
  while (<SYSINFO>)
  {
    #print $_, br() ;
    # Each important line should look like:
    #   tag: value
    # If that's the case, separate the two fields and save the
    # value if the tag is one we are interested in.
    if ( my ($tag, $value) = m/^\s*(\S*)\s*:\s*(.*)\s*$/ )
    {
      $refStationData{'lat'} = $value if ($tag eq "Lat" ) ;

      $refStationData{'lon'} = $value if ($tag eq "Lon" ) ;
      $refStationData{'height'} = $value if ($tag eq "Height" ) ;

      $refStationData{'x'} = $value if ($tag eq "X" ) ;
      $refStationData{'y'} = $value if ($tag eq "Y" ) ;
      $refStationData{'z'} = $value if ($tag eq "Z" ) ;
      $refStationData{'rtcmid'} = $value if ($tag eq "RTCM_ID" ) ;
      $refStationData{'cmrid'} = $value if ($tag eq "CMR_ID" ) ;
      $refStationData{'rtcmname'} = $value if ($tag eq "StationName" ) ;
      $refStationData{'rtcmdesc'} = $value if ($tag eq "StationDesc" ) ;
    }
  }
  close( SYSINFO ) ;

  # Because we commonly want to display lat and lon in DegreeMinSec
  # format, we do the conversion here in a single place and return
  # those fields in the hash.
  
  my $latitude = $refStationData{'lat'} ;
  my $longitude = $refStationData{'lon'} ;

  my ($deg, $min, $sec, $dir ) ;

  $dir = 'N' ;
  if ($latitude < 0.0) 
  {
    $latitude = - $latitude ;
    $dir = 'S' ;
  }

  # Round the latitude to nearest 0.00001 second.  This works
  # in conjunction with the '$sec = int($sec...) line a little
  # bit below here, which truncates to nearest 0.00001 second.
  $latitude += (0.000005 / 3600) ;

  $deg   = int( $latitude ) ;
  $latitude = ($latitude - $deg) * 60 ;
  $min   = int( $latitude ) ;
  $sec   = ($latitude - $min) * 60 ;
  $sec   = int($sec * 100000.0)/100000.0 ;

  $refStationData{'latdeg'} = $deg ;
  $refStationData{'latmin'} = $min ;
  $refStationData{'latsec'} = $sec ;
  $refStationData{'latns'}  = $dir ;

  $dir = 'E' ;
  if ($longitude < 0.0) 
  {
    $longitude = - $longitude ;
    $dir = 'W' ;
  }

  # Round the longitude to nearest 0.00001 second.  This works
  # in conjunction with the '$sec = int($sec...) line a little
  # bit below here, which truncates to nearest 0.00001 second.
  $longitude += (0.000005 / 3600) ;

  $deg   = int( $longitude ) ;
  $longitude = ($longitude - $deg) * 60 ;
  $min   = int( $longitude ) ;
  $sec   = ($longitude - $min) * 60 ;
  $sec   = int($sec * 100000.0)/100000.0 ;

  $refStationData{'londeg'} = $deg ;
  $refStationData{'lonmin'} = $min ;
  $refStationData{'lonsec'} = $sec ;
  $refStationData{'lonew'}  = $dir ;

  %{$_[0]} = %refStationData ;

} # end of getRefStationData() 



########################################################################
sub updateRefStationData
########################################################################
# Updates the reference station configuration based on the settings
# pass in via a hash reference parameter.  The hash contains settings
# for:
#   lat lon height 
#   x y z
#   rtcmid
#   cmrid
#   rtcmname
#   rtcmdesc
# A second parameter $_[1] determines whether we put xyz or llh into
# the configuration file.  zero uses llh.  non-zero uses xyz.
########################################################################
{
  # Create a local copy of the hash.
  my %refStationData = %{$_[0]} ;

  my $xyz = $_[1] ; # True if we want XYZ in the config file.

  #print "Updating RefStation data\n" ;

  #for my $p (keys %refStationData)
  #{
  #  print "$p = $refStationData{$p}\n" ;
  #}

  # Update the reference station configuration file with the new data.
  # Open the existing file, and copy it line-by-line into a
  # copy in /tmp.  Leave out any lines containing the information
  # the user has just entered.
  open( NEWCONF, '>/tmp/ref_stn.config' ) ;

  # Copy the information from the existing config file,
  # leaving out the item that is to be deleted.
  if (open( OLDCONF, '/etc/sysconfig/ref_stn.config' ))
  {
    while (<OLDCONF>)
    {
      if ( ! m/^\s*(Lat|Lon|Height|X|Y|Z)\s*:/
        && ! m/^\s*(RTCM_ID|CMR_ID|StationName|StationDesc)\s*:/)
      {
        print NEWCONF $_ ;
      }
    }
    close( OLDCONF ) ;
  }

  # Add the updated information lines to the end of the new file.
  if ($xyz)
  {
    printf NEWCONF "X: %12.4f\n", $refStationData{'x'} ;
    printf NEWCONF "Y: %12.4f\n", $refStationData{'y'} ;
    printf NEWCONF "Z: %12.4f\n", $refStationData{'z'} ;
  }
  else
  {
    printf NEWCONF "Lat:    %17.12f\n", $refStationData{'lat'} ;
    printf NEWCONF "Lon:    %17.12f\n", $refStationData{'lon'} ;
    printf NEWCONF "Height: %8.4f\n", $refStationData{'height'}  ;
  }

  print NEWCONF "RTCM_ID: ", $refStationData{'rtcmid'},"\n",
                "CMR_ID:  ", $refStationData{'cmrid'},"\n",
                "StationName: ", $refStationData{'rtcmname'},"\n", 
                "StationDesc: ", $refStationData{'rtcmdesc'},"\n" ;

  close( NEWCONF ) ;

  # Copy the new file on top of the old one.
  system('sudo cp /tmp/ref_stn.config /etc/sysconfig/ref_stn.config' ) ;

  # Tell the sysinfo system to load in the revised data.
  system( 'sudo /usr/local/fireball/sysinfo -m' ) ;

} # end of updateRefStationData() 



########################################################################
sub getNtpServers
########################################################################
# Read the configuration file for the NTP client setup, and extract
# a list of the current NTP servers.  Return a @list containing this
# data.  If the NTP client is disabled, then the list will be empty.
########################################################################
{
  my @servers ;

  # Read configuration from /etc/sysconf/ntp.conf, looking for lines
  # that define ntp servers, that aren't part of the default Fireball
  # setup.  These will be of the form
  #       server  IP.or.DNS.name      # External Server
  if ( open( NTPCONF, '/etc/sysconfig/ntp.conf' ) )
  {
    while (<NTPCONF>)
    {
      # If this is a matching line, grab the second field which
      # contains the name of the server.  Add that name to our
      # list.
      push( @servers, ((split())[1]) ) 
        if (m/^server .*# External Server$/) ;
    }
    close( NTPCONF ) ;
  }

  return @servers ;

} # end of getNtpServers



########################################################################
sub saveNtpServers
########################################################################
# Passed a list of servers which should be installed in the
# ntp.conf and step-tickers configuration files.  
# Rewrites (or deletes) the files, and then restarts the NTP subsystem.
#
# Note that if the list is empty, then the NTP system is "disabled".
# That removes step-tickers and puts no 'server' lines in the
# ntp.conf.
########################################################################
{
  my @servers = @_ ; # Local copy for code readability.

  # Re-generate the file /etc/sysconfig/ntp.conf
  # We read through the original file, transfering and filtering to a
  # new version in /tmp.  
  # Lines of the form 'server .... # External Server' are removed and
  # replaced with data from the CGI parameters.
  open( OLDCONF, '/etc/sysconfig/ntp.conf' ) ;
  open( NEWCONF, '>/tmp/ntp.conf' ) ;

  while (<OLDCONF>)
  {
    print NEWCONF $_  if ( ! m/^server .*# External Server$/) ;
  }

  # Now add the new external servers (if any) at the end of the file.
  for my $server (@servers)
  {
    print NEWCONF "server $server # External Server\n" ; 
  }

  close( NEWCONF ) ;
  close( OLDCONF ) ;

  # Copy the new configuration file into the real config directory.
  # Must use sudo as sysconfig is writeable only by root.
  system( 'sudo cp /tmp/ntp.conf /etc/sysconfig/ntp.conf' ) ;
  unlink( '/tmp/ntp.conf' ) ;


  # The first time server listed is used as the "stepticker", which
  # means it's the server we'll go to to jamsync time at boot time.
  # If this server is defined, we'll record it in
  # /etc/sysconfig/step-tickers, otherwise we'll remove that file.

  if ($servers[0])
  {
    open( NEWCONF, '>/tmp/step-tickers' ) ;
    print NEWCONF "$servers[0]\n" ;
    close NEWCONF ;
    system( 'sudo cp /tmp/step-tickers /etc/sysconfig/step-tickers' ) ;
    unlink( '/tmp/step-tickers' ) ;
  }
  else
  {
    if ( -f '/etc/sysconfig/step-tickers' )
    {
      system( 'sudo rm /etc/sysconfig/step-tickers' ) ;
    }
  }


  # Determine the process id of the current ntpd process.
  # Since there isn't a /var/run file, the only way to do this
  # is to run the 'ps' command and grep for the process name.
  my $ntpPID = 0 ;
  open( PS, '/bin/ps |' ) ;
  while (<PS>)
  {
    my ($pid, undef, undef, undef, $process, undef) = split ;
    if ( $process eq 'ntpd' ) 
    {
      $ntpPID = $pid ; 
      last ;
    }
  }
  close( PS ) ;

  if ($ntpPID)
  {
    system( "/usr/bin/sudo kill $ntpPID" ) ;
  }

} # end of saveNtpServers()



########################################################################
sub getIoStatus
########################################################################
# Returns an array of client description hash references,
# describing any current I/O streaming activity.
# Each element of the returned array is a reference to a hash,
# corresponding to a single active "client" connection.  
#
# A serial port can only have a single connection.  PPP ports don't
# count, and won't be listed.
#
# TCP or UDP ports can have multiple remote clients.  Each of those
# counts as a separate client so there can be multiple entries for a
# single TCP or UDP port.  
#
# For each client, the hash describing it contains elements:
#   'name'     : SerialPortN, TcpPortNNNN or UdpPortNNNN
#   'protocol' : Either serial, tcp, or udp.  Derivable from the 'name',
#                but its easier to just preserve this information.
#   'service'  : What's being streamed.
#                   rtcm, rt17, met, info, trimcom, test, metdata, etc.
#   'ip'       : For TCP or UDP clients, the IP address of the remote
#                connection point.
#
# Calling convention probably like this:
#   my @clients = getIoStatus() ;
#
# Access an element like this:
#
#   $clients[1]{'name'} 
#
########################################################################
{

  # Use socket 8086 on localhost.  For some reason localhost doesn't
  # always resolve, so we have to use a numeric value.
  my ($remote, $port) = ( '127.0.0.1', 8086 ) ;
  my $destaddr = sockaddr_in( $port, inet_aton( $remote ) ) ;

  # Connect to that destination address, so we can read the data
  # from the socket.
  socket( SOCK, 
          PF_INET, 
          SOCK_STREAM, 
          getprotobyname('tcp') 
        ) || die "socket: $!";
  connect( SOCK, $destaddr ) ;

  # The format coming back from the gpssd info service looks like thie
  # following.  I haven't found a document that describes this, so
  # everything done below is just inference based on apriori knowledge.

  # *********  Stream Info request # 759
  # ID Protocol Port Status Clients Baud  S/N  Timeout Type Rate  Options
  # ----------------------------------------------------------------------
  # 02: serial 00002 0102  001/010 115200 00757 000     met 01 Hz
  # 04:    tcp 08080 0302  000/010 000000 00004 000    rt17 01 Hz 00
  # 05:    tcp 08086 0401  001/010 000000 00005 045 service info
  # 06:    tcp 08100 0502  000/001 000000 00006 000 trimcom 01 Hz
  # 07:    tcp 04001 0601  000/010 000000 00007 000 service test
  # 09:    tcp 08099 0801  000/010 000000 00758 000 metdata 10 Hz
  #   Configured formatters: 7
  #   Configured streams...: 9
  # 
  # *********  Client Info
  # ID Socket Address        S/N   Timeout Connected Written  Dropped # bytes/s
  # ----------------------------------------------------------------------
  # 05   015  c0a88e07:33295 00759 000045  00000000  00000000 00000000 000000
  #   Configured tcp clients...: 1
  # 
  # ID Socket Address        S/N   Timeout Connected Written  Dropped bytes/s
  # ----------------------------------------------------------------------
  #   Configured udp clients...: 0
  # 
  # ID Socket Address        S/N   Timeout Connected Written  Dropped bytes/s
  # ----------------------------------------------------------------------
  # 02   013  00000000:00013 00757 -00006  00000006  00000000 00000000 000000
  #   Configured serial clients...: 1


  # In order to parse that, we go through the output line by line.
  # Anything that starts with a digit-digit-colon is regarded as a
  # stream definition.
  # Anything that starts with a digit-digit-space is regarded as an
  # active client connection, with the first field referencing a
  # particular stream.  
  # We create a database of all of the streams (in %streams) and then
  # a list of all of the active clients worthy of showing to the user.

  my %streams ;   # A hash of hashes, indexed by stream ID
  my @clients ;   # An array of hash references

  while (<SOCK>)
  {
    #print $_ ;
    my @fields = split ; # Split into an array of strings.

    # If the line starts with digit-digit-colon...
    if (m/^\d\d:/)
    {
      # It's a stream definition.
      # Extract and adjust the pertinent information.
      my $id = $fields[0] ;
      $id =~ s/:// ;

      my $protocol = $fields[1] ;

      my $port = $fields[2]+0 ;
      ++$port if ($protocol eq 'serial') ;

      my $service = $fields[8] ;
      $service = $fields[9] if ( $service eq 'service' ) ;

      # Put the information into a hash of hashes
      $streams{$id} = { 'protocol' => $protocol ,
                        'port' => $port,
                        'service' => $service,
                      } ;
      next ;
    }

    if (m/^\d\d /)
    {
      # This is a client/connection description.
      # Extract relevant data from the line..
      my $id = $fields[0] ; 

      # Get the client's IP address.  The field is of the form
      # c0a88e07:33295, where the first part is the IP address
      # and the send is (?perhaps?) the port number.
      my $hexIP = $fields[2] ; 

      $hexIP =~ s/:.*// ; # Drop the port number portion.

      # Convert the two-character hex octets into standard
      # 155.63.21.25 format.
      my $ip = hex( substr( $hexIP, 0, 2 ) ) ;
      $ip .= '.' ;
      $ip .= hex( substr( $hexIP, 2, 2 ) ) ;
      $ip .= '.' ;
      $ip .= hex( substr( $hexIP, 4, 2 ) ) ;
      $ip .= '.' ;
      $ip .= hex( substr( $hexIP, 6, 2 ) ) ;

      my $service = $streams{$id}{'service'} ;
      my $protocol = $streams{$id}{'protocol'} ;
      my $port = $streams{$id}{'port'} ;
      my $name ;
      $name = "SerialPort$port" if ($protocol eq 'serial') ;
      $name = "TcpPort$port" if ($protocol eq 'tcp') ;
      $name = "UdpPort$port" if ($protocol eq 'udp') ;

      # Add this client's information to the list of clients.
      # Don't include the service generating this list in the listing.
      if ($service ne 'info')
      {
        my %client = ( 'name' => $name,
                       'protocol' => $protocol,
                       'port' => $port,
                       'service' => $service,
                       'ip' => $ip,
                     ) ;
        # put a reference to that hash into the list we'll be returning.
        push @clients, \%client ;
      }
    }
  }
  close SOCK ;

  # Sort the list of clients by name
  @clients = sort( {${$a}{'name'} cmp ${$b}{'name'}} @clients ) ;

  return @clients ;

} # enf of getIoStatus()





# Return a '1' as the final operation in this library, to be sure
# that whoever imports it sees success.
1 ;
########################################################################
# End of file Fireball.pm
########################################################################
