#!/usr/bin/perl -w
# Copyright (c) 2003 Trimble Navigation Limited
# $Id: deleteFile.cgi,v 1.6 2003/06/13 16:48:47 tom Exp $
# $Source: /home/CVS/fireballTop/fireballFS/root/usr/local/apache/htdocs/fireball/perl-scripts/fileaccess/deleteFile.cgi,v $

########################################################################
# deleteFile.cgi
#
# Script for deleting a logged data file.
#
# This script is called with two CGI parameters: 
#       
#   ?delete=filepath
#
#   ?sort=[date|size|name]
#
# The Logged Data files are rooted at /var/gps/.  
# This request deletes the named file, and then returns to 
# display the directory that contained that file, including a message
# showing what happened.
#
# If the filepath specifies a directory then the entire directory rooted
# there is deleted.
#
# When this script finishes deleting a file, it immediately re-invokes
# the directory display page at perl-scripts/dataFiles.cgi.  It sends
# that script a ?dir= cgi parameter (derived from the file that has been
# deleted) and a '?sort=' parameter, as was fed into the script.  This
# lets us ping-pong back and forth between the directory page, and
# the delete script without losing track of where we are and how the
# directory display is sorted.
#
########################################################################

use strict ;
use CGI qw/:standard/;

use lib '..' ;
use Fireball ;

startPage( 'File Delete' ) ;

my $Rootdir='/var/gps' ;



#for my $p (param())
#{
#  print $p, '=', param($p) , br() ;
#}



# Determine how the entries should be sorted.  Default is by name.
# This value should come in from 
my $sortBy = param('sortBy') ;
if ( ! $sortBy )
{
  $sortBy = 'name' ;
}


# If this request is to delete a file, handle that before displaying
# the directory again.  Indicate what was deleted.
# Default value is to show that something went wrong.
my $message = 'Missing parameter to deleteFile.cgi' ;

# Set this dirpath below to the directory path to display after
# the file(s) are deleted.
my $dirpath ;

if ( my $file = param( 'delete' ) )
{
  # Determine the path of the file to be deleted.
  $dirpath = $file ;
  $dirpath =~ s|/[^/]*$|/| ; # Drop everything after last slash.

  if ($file eq '/')
  {
    print h2( $message = "Can't delete '/' directory." ) ;
  }
  elsif ( $file =~ m|/$| )
  {
    # The filename ends in a '/' so this is really a directory delete
    # (or, perhaps, a symlink pointing to a directory).

    $file =~ s|/$|| ; # Drop the final '/' so -l test below works.
    if ( -l $Rootdir.$file )
    {
      print h2( $message = "Symlink $file has been deleted." ) ;
      unlink( $Rootdir.$file ) ;
    }
    else
    {
      # Either we are deleting an empty or a full directory.  Could do
      # this from perl, but the easiest way is just to call the Unix rm
      # -fr command.
      print h2( $message = "Directory $file has been deleted." ) ;
      system( "rm -fr $Rootdir$file" ) ;
    }
    $dirpath =~ s|[^/]*/$|| ; # Remove last element/ from path.
  }
  else
  {
    print h2( $message = "File $file has been deleted." ) ;
    unlink $Rootdir.$file ;
  }
  print br(),br() ;
}

# Immediately switch back to the directory page.  
# Pass parameters dir, sort, and msg.
# Since msg can have spaces, have to http encode it. (space to %20,etc.)
my $URL = "/perl-scripts/dataFiles.cgi?dir="
          .CGI::escape( $dirpath )
          ."&sort=$sortBy&msg="
          .CGI::escape( $message ) ;
print <<EOF ;
<script>
  document.location = "$URL" ;
</script>
EOF

print end_html() ;

########################################################################
# end of script deleteFile.cgi
########################################################################
