#!/usr/bin/perl -w
# Copyright (c) 2003 Trimble Navigation Limited
# $Id: setWaas.cgi,v 1.5 2003/06/12 22:45:40 tom Exp $
# $Source: /home/CVS/fireballTop/fireballFS/root/usr/local/apache/htdocs/fireball/perl-scripts/gpsadmin/setWaas.cgi,v $

########################################################################
# setWaas.cgi
#
# Sets the controls for the tracking and usage of WAAS satellites.  
# This script is called with parameters describing the desired 
# controls, typically in response to the user clicking OK in the
# script /perl-scripts/waas.cgi.
#
# The control states in the configuration file are adjusted to match 
# the parameters.  Likewise, Stinger is informed of any changes.

# The parameters are:
#   WaasControl : 'On' or 'Off'
#   Enable120, Enable121 : will be "1" if set, and not submitted if not.
#                          Thus we can just test if the parameter is
#                          defined to see if a satellite is enabled.
#   Ignore120, Ignore121 : will be "1" if set, and not submitted if not.
#                          Thus we can just test if the parameter is
#                          defined to see if a satellite is
#                          ignoreHealth.
#
# The parameter names are based on the satellite numbering scheme used
# by Stinger.  PRNs 120 through 138 are Waas/Egnos style satellites.
# These are mapped to labels like 'WAAS POR' on the displayed page.  IF
# WE PUBLISH THIS CGI INTERFACE, we'll need to rethink that naming.
# It's too implementation dependent.
#
# Stinger's Waas control interface relies on several commands:
#   put waas_mode on    (or 'off')
#       Enables or disables tracking of waas signals
#
#   get waas_mode
#       Returns the appropriate 'put waas_mode' string.
#
#   put waas_sv_masks D D E D D I D D D D D D D D D E D D D  
#       Shows the Enable/Disable state of each satellite.
#       First symbol is PRN 120.  Last is 138.  There are
#       nineteen Waas style satellites.
#
#   get waas_sv_masks
#       Returns the current 'put waas_sv_masks' string.
#
# The 'put' strings are saved in /etc/sysconfig/stinger.cfg to provide
# non-volatile storage.
# 
# The Ignore health controls might not be needed in the long run.  See
# Stuart Riley for details on that.  For the moment I've left
# IgnoreHealth installed pending further discussions.
########################################################################


use strict ;
use CGI qw/:standard/;
use IO::Socket;

use lib '..' ;
use Fireball ;

########################################################################
# Begin formatting the return document.
########################################################################
startPage( 'Set Waas' ) ;


#print '<PRE>' ;
#foreach my $p (param())
#{
#  print "$p = ", param($p), "\n" ;
#}
#print '</PRE>' ;

my $enable = param('WaasControl') ;
if ( $enable eq 'On' or $enable eq 'Off' )
{
  my $sun = sockaddr_un('/var/stinger/config')|| die "no config";
  socket(SOCK, PF_UNIX, SOCK_STREAM, PF_UNSPEC)|| die "socket: $!";
  connect(SOCK,$sun)|| die "connect: $!";
  SOCK->autoflush(1);
  
  my $line1 = 'put waas_mode '
             .( ( $enable eq 'On') ? 'on' : 'off' )
             ."\n" ;

  #print $line1 ;
  print( SOCK  $line1 ) ;
  $_ = <SOCK> ;
  #print $_ ;

  my $line2 = 'put waas_sv_masks' ;
  for (my $i = 120; $i <= 138 ; ++$i )
  {
    $line2 .= param('Ignore'.$i)  
              ? ' I' 
              : param('Enable'.$i) 
                ? ' E' 
                : ' D' ;
  }

  $line2 .= "\n" ;
  #print $line2 ;
  print( SOCK  $line2 ) ;
  $_ = <SOCK> ;
  #print $_ ;

  close(SOCK) ;
  #print '</PRE>' ;
  
  # Update the stinger configuration file to make this change
  # persist across power cycles.
  open( NEWCONFIG, '>/tmp/stinger.cfg' ) ;

  # Read and transfer the old data to the new file.  
  # Leave out the commands we are trying to modify.
  if ( open( OLDCONFIG, '/etc/sysconfig/stinger.cfg' ) )
  {
    while (<OLDCONFIG>)
    {
      next if (/put waas_sv_masks/) ;
      next if (/put waas_mode/) ;
      print NEWCONFIG $_ ;
    }
  }
  close OLDCONFIG ;


  print NEWCONFIG $line1 ;
  print NEWCONFIG $line2 ;

  close NEWCONFIG ;

  system( 'sudo cp /tmp/stinger.cfg /etc/sysconfig/stinger.cfg' ) ;

  print h2( "New Waas controls have been set." ) ;

  # After a timeout, go back to main satellite screen.
  print <<EOF;
    <Script>
    parent.loadAfterDelay( "/perl-scripts/satGeneral.cgi", 1000) ;
    </Script>
EOF
}
else
{
  print h2( "Bad Waas control parameters." ) ;
  print <<EOF;
    <Script>
    parent.loadAfterDelay( "/perl-scripts/waas.cgi", 1000) ;
    </Script>
EOF

}

print end_html() ;

########################################################################
# end of file setWaas.cgi
########################################################################
