#!/usr/bin/perl -w
# Copyright (c) 2003 Trimble Navigation Limited
# $Id: pps.cgi,v 1.7 2003/06/12 22:45:39 tom Exp $
# $Source: /home/CVS/fireballTop/fireballFS/root/usr/local/apache/htdocs/fireball/perl-scripts/pps.cgi,v $

########################################################################
# pps.cgi
#
# Puts up a form that allows user to view and enter the controls for the
# the Pulse-Per-Second feature.
# Can be used to enable or disable the pulses and to set the polarity of
# the leading edge.
#
# No CGI parameters are expected.  We just display the current state
# of the controls in an HTML form.  When the suer clicks OK, we submit
# the data to a separate gpsadmin-secured script setPps.cgi.
#
# The pps configuration is saved in a line from 
# /etc/sysconfig/stinger.cfg.  The line is of the forms: 
#
#   put pps_mode on pos
#   put pps_mode off neg
#
# That same format is used for direct control of Stinger using the
# Stinger Info connections.
#
# The sysinfo system is used to query the current setting of the
# information using 'sysinfo -p pps_mode', which returns a 
# "put pps_mode command" string showing the current settings.
#
########################################################################


use strict ;
use CGI qw/:standard/;
use Fireball ;

titleWithHelp( 'PPS (Pulse-Per-Second) Controls',
                         '/ppsInfo.html'
                       ) ;


# Determine the current setting of the pps controls.
# This is done by querying the gpsinfo database via 'sysinfo', which
# must be run as root via sudo.
# The response will be of the form:
#   put pps_mode [on|off] [pos|neg]

my (undef, undef, $pps_enable, $pps_slope) 
   = split( " ",  
            qx(  /usr/bin/sudo /usr/local/fireball/sysinfo -p pps_mode )
          ) ;

param( 'PpsEnable', ($pps_enable eq 'on' ? 'On' : 'Off' ) ) ;
param( 'PpsSlope' , ($pps_slope eq 'pos' ? 'Positive' : 'Negative' ) ) ;


print startform( -name=>'theForm', 
                 -action=>'/perl-scripts/gpsadmin/setPps.cgi'
               ) ;


my @PPSControl = radio_group( -name=>'PpsEnable', 
                              -value=>[ 'On', 'Off' ],
                              -default=>'On',
                            ) ;

my @PpsSlope   = radio_group( -name=>'PpsSlope', 
                              -value=>[ 'Positive', 'Negative' ],
                              -default=>'Positive',
                            ) ;
print table( Tr(
                th( 'PPS Enable:' ),
                td( $PPSControl[0], 
                    br(), 
                    $PPSControl[1]  
                  ),
                td( {width=>'36'} ),
                th( 'PPS Slope:' ),
                td( $PpsSlope[0], 
                    br(), 
                    $PpsSlope[1]  
                  ),
               ),
           ) ;

#print br(), b('Note:'), "Ascii time tags for 1PPS pulses must be enabled
#separately in the I/O Configuration pages." ;

print hr() ;

# Put up the buttons.
print button( -name=>'OK', 
              -value=>'OK',
              -onClick=>'parent.submitForm( document.theForm, "" ) '
            ) ;
print '&nbsp;' ;
print button( -name=>'Cancel', 
              -value=>'Cancel',
              -onClick=>'parent.loadDataFrame( 
                           "/perl-scripts/rconfigDisplay.cgi" 
                                             )'
            ) ;


print endform() ;

 
print end_html() ;

########################################################################
# end of file pps.cgi
########################################################################
