#!/usr/bin/perl -w
# Copyright (c) 2003 Trimble Navigation Limited
# $Id: rstatusActivity.cgi,v 1.30.2.2 2003/11/18 02:11:28 tom Exp $
# $Source: /home/CVS/fireballTop/fireballFS/root/usr/local/apache/htdocs/fireball/perl-scripts/rstatusActivity.cgi,v $

########################################################################
# File rstatusActivity.cgi
# 
# Perl Script to format a page showing current "activities" in a 
# Fireball.  This includes a summary of how many satellites are being
# tracked, what data logging is going on, what output streams are
# active, etc.
#
# Currently this is just a mockup of the page with some samples of
# what might be included.  None of the data is real except for the
# timestamp, and the satellites being tracked.
#
########################################################################

use strict ;

use CGI qw/:standard/;
use IO::Socket;

use Fireball ;

titleWithHelp( 'Receiver Status - Activity',
                         '/activityInfo.html'
                       ) ;


########################################################################
# List satellites being tracked.
########################################################################

# Determine which satellites are being tracked.  Ask the sysinfo
# utility for the tracking status report.
open( SYSINFO, 
      '/usr/bin/sudo /usr/local/fireball/sysinfo -p tracking_status |');

# Response is twelve lines, one per channel of the form
#   0 :  15 51.7 93.3 D2FF   8001 50.8 41.8 35 2.8
#  Chn  PRN Elev Azm  l1Stat     
# Or (if channel is unused)
#   0 : idle
# Scan the list of Sv information, and extract a list
# of satellite's being tracked.  The definition of tracked is
# that the L1 status indicates measurements are being made <=> the
# 64-bit in the L1 status is set.
my @svList ;
for ( my $i = 0 ; $i < 12 ; $i++ )
{
  if ( ($_ = <SYSINFO>) !~ /idle/ )
  {
    #print $_, "<BR>" ;
    ( undef, undef, my $prn, undef, undef, 
      my $l1stat, my $l2stat, undef       ) = split( ' ' ) ; 

    if ( hex($l1stat) & 64)
    {
      push @svList, $prn ;
    }
  }
  else
  {
    #print $_, "<BR>" ;
  }
}

close( SYSINFO ) ;

my $indent='&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;' ;

print "<b>Satellites Tracked:</b>" ;

# Emit sorted list of PRN numbers. 
# Sort the SV list numerically, replace the WAAS prns with their
# textual names, and output the list, separated by spaces.  
@svList = sort( {$a <=> $b} @svList ) ;
for my $prn (@svList)
{
      if    ($prn == 120) { $prn = 'EGNOS-AOR-E'; } 
      elsif ($prn == 122) { $prn = 'WAAS-AOR-W' ; }
      elsif ($prn == 129) { $prn = 'MSAS-1'        ; } 
      elsif ($prn == 131) { $prn = 'EGNOS-IOR'  ; }
      elsif ($prn == 134) { $prn = 'WAAS-POR'   ; }
      elsif ($prn == 137) { $prn = 'MSAS-2'        ; }
}

$svList[0] = 'None' if (! @svList) ;
print table( Tr( td( "&nbsp;&nbsp;&nbsp;" ),
                 td( code( join( ' ', @svList ) ) )
               )
           ) ;


########################################################################
# List active data logging sessions.
########################################################################
print "<p><b>Data Logging:</b><br>";
chdir '/var/run' ;
my @sessions = glob( "gpslog.*" ) ;
if (@sessions)
{
  print '<Table>' ;
  for my $sess (@sessions)
  {
    my ( undef, undef, $sessname, undef, undef, undef, undef, undef,
         $file, undef
       ) = split( " ", qx( cat $sess ) ) ;
    $file =~ s|^/var/gps|| ;
    print "<Tr><td>&nbsp;&nbsp;&nbsp;</td><td>Session</td><td>",
          $sessname,
          "</td><td>logging to ",
          $file,
          "</td></tr>"
          ;
  }
  print "</Table>" ;
}
else
{
  print "${indent}No Sessions Active<br>" ;
}


########################################################################
# List active I/O ports
########################################################################
print '<br><b>Input/Output:</b><table>' ;
my @clients = getIoStatus() ;
# Access an element like this:
#   $clients[1]{'name'} 

if (@clients)
{
  for my $i (0..$#clients)
  {
    my $status ;
    my $name = $clients[$i]{'name'} ;
    my $port = $clients[$i]{'port'} ;
    my $protocol = $clients[$i]{'protocol'} ;
    $protocol = 'Tcp' if $protocol eq 'tcp' ;
    $protocol = 'Udp' if $protocol eq 'udp' ;
    my $service = $clients[$i]{'service'} ;
    $service = 'CMR' if $service eq 'cmr' ;
    $service = 'RTCM' if $service eq 'rtcm' ;
    $service = 'Rt17' if $service eq 'rt17' ;
    my $ip = $clients[$i]{'ip'} ;
    $status = "--$name $port $protocol $service $ip" ;

    next if ($service eq 'metdata') ;

    if ($service eq 'met')
    {
      $status = "Met/Tilt interface active on Serial Port $port" ;
    }
    elsif ($protocol eq 'serial')
    {
      $status = "Streaming $service over Serial Port $port" ;
    }
    else
    {
      $status = "Streaming $service to $ip over $protocol Port $port" ;
    }


    print Tr( td( '&nbsp;&nbsp;&nbsp;' ),
              td( $status ) 
            ) ;
  }
}
else
{
  print Tr( td( '&nbsp;&nbsp;&nbsp;' ),
            td( 'There are no currently active I/O Streams.' )
          ) ;
}
print '</table>' ;


my ( $primaryV, $secondaryV, $temperature ) = (0,0,0);

if ( -x '/usr/local/fireball/voltage' )
{
  open( VOLTS, '/usr/local/fireball/voltage -a |' ) ;
  $primaryV = <VOLTS> ;
  $secondaryV = <VOLTS> ;
  $temperature = <VOLTS> ;
  close VOLTS ;
  $primaryV = (split( ' ', $primaryV ))[3] ;
  $secondaryV = (split( ' ', $secondaryV ))[3] ;
  $temperature = (split( ' ', $temperature ))[2] ;
}

print '<br><b>Power Supply Info:</b>' ;
print ' <blink>FAKE</blink>' if ( ! -x '/usr/local/fireball/voltage' ) ;

print table( Tr( td( '&nbsp;&nbsp;&nbsp;' ),
                 td( 'Primary Voltage:' ),
                 td( sprintf( "%.3f Volts", $primaryV ) ) 
               ),
             Tr( td( '&nbsp;&nbsp;&nbsp;' ),
                 td( 'Secondary Voltage:' ),
                 td( sprintf( "%.3f Volts",  $secondaryV ) ) 
               ),
             Tr( td( '&nbsp;&nbsp;&nbsp;' ),
                 td( 'Temperature:' ),
                 td( "$temperature&deg; C" ) 
               )
           ) ;



########################################################################
# Create Timestamp:
# 2001 Sep 6 - Thu of GPS Week 1130 - 17:46:52 GMT 
print hr() ;
print timestamp() ;

# Delay a bit and then refresh this page.  We try to make this happen on
# exact 5 second boundaries.  If this time was an exact 5 second time,
# then we wait 4000 msec to allow a little bit of time for
# transmission and processing.  The range of delays is from 2000 to 
# 6000.
my $delay = 6000 - ( (time()+2) % 5 ) * 1000 ;
print "
<script>
  // Reload this page every five seconds
  parent.loadAfterDelay( window.location.href, $delay) ;
</script>
" ;


print end_html() ;

########################################################################
# end of file rstatusActivity.cgi
########################################################################
