#!/usr/bin/perl -w
# Copyright (c) 2003 Trimble Navigation Limited
# $Id: sessions.cgi,v 1.42.2.1 2003/10/30 22:25:11 tom Exp $
# $Source: /home/CVS/fireballTop/fireballFS/root/usr/local/apache/htdocs/fireball/perl-scripts/sessions.cgi,v $

########################################################################
# sessions.cgi
#
# Script for displaying a summary of the current data logging sessions
# in fireball.  Contains buttons for invoking screens for creating
# new sessions and editing existing ones.
#
# This is an auto-refreshing screen because it shows realtime status of
# the system.  A configuration file /etc/sysconfig/sessions contains the
# descriptions of all data logging sessions.  This is read and parsed
# into various hashes.  That data is combined with runtime information
# to determine which sessions are active, pending, disabled, etc.
########################################################################

use strict ;
use CGI qw/:standard/;
use Fireball ;

titleWithHelp( 'Data Logging Status',
               '/sessionsInfo.html'
             ) ;

# Get the time of day in GPS seconds.  Includes a 13 second offset for
# the UTC to GPS leapseconds.  315964800 converts from Unix time to
# gps time (01Jan1970 to 06Jan1980).
my $gpsTime = time() - 315964800 + 13 ;


# Read in the data from the sessions configuration file.
my %sessData ;
getSessionData( \%sessData ) ;


# Read in the descriptions of any active sessions, as defined by
# files of the form /var/run/gpslog.*.  @ActiveSessions will contain
# an array of the contents of these files, which can be searched for
# specific session names, and the parameters of the active session(s)
# associated with those names.
my @ActiveSessions = () ;
for my $f (glob '/var/run/gpslog.*')
{
  # Open and read that file.
  # Note that though the file $f existed above (glob found it!) it
  # may have gone away before we got around to reading it.  So test
  # if the open succeeds here:
  if ( open( LOG,  $f ) )
  {
    my $line = <LOG> ; # Get first line

    # The file should always have a line, but sometimes it doesn't
    # during testing.  If not, it will be empty and we'll ignore it.
    if ($line)
    {
      # Add to list of sessions if the file contained anything
      push @ActiveSessions, $line ;
    }
    close LOG ;
  }
}


print startform() ;

# All of the items displayed on this page are center aligned on the
# page, so we define a 'div' section to cover them all.
print '<div align="center">' ;


print '<p><Table border>' ;
print Tr( th( '&nbsp;Session&nbsp;<BR>Name' ),
          th( '&nbsp;Schedule&nbsp;' ),
          th( 'Duration<BR><font size="-1">[minutes]</font>' ),
          th( '&nbsp;Status&nbsp;' ),
          th( '&nbsp;Enable&nbsp;' )
        ) ;

foreach my $sessname (sort keys( %sessData))
{
  my $enable = $sessData{$sessname}{'enable'} ;
  my $sched = $sessData{$sessname}{'sched'} ;
  my $time = $sessData{$sessname}{'time'} ;
  my $duration = $sessData{$sessname}{'duration'} ;

  print "<tr align='center'>" ;

  my $link= "javascript:parent.loadDataFrame('editSession.cgi?restart="
          .CGI::escape($sessname)
          ."')" ;
  print td( '&nbsp;',a( {href=>$link}, $sessname), '&nbsp;' ) ;

  if ($sched eq "manual") 
  { 
    print td( '&nbsp;Manual&nbsp;' ) ;
  }
  elsif ($sched eq "cont") 
  { 
    print td( "&nbsp;Continuous&nbsp;" ) ;
  }
  elsif ($sched eq "daily") 
  { 
    print td( sprintf( "&nbsp;Daily at %02d:%02d&nbsp;", 
                       int( $time / 3600 ) ,
                       int( $time % 3600 / 60 ) 
                     )
            ) ;
  }
  else
  {
    # Once only session
    my (undef,$min,$hour,$day,$month,$year,undef)
      = gmtime( $time + 315964800 ) ;

    $month = ( 'Jan','Feb','Mar','Apr',
               'May','Jun','Jul','Aug',
               'Sep','Oct','Nov','Dec' )[$month] ;

    print td( sprintf( '&nbsp;%04d/%3s/%02d@%02d:%02d&nbsp;', 
                       $year+1900, $month, $day, $hour, $min
                     )
            ) ;
  }

  print td( $duration ) ;

  # Print the status
  my $status = "" ;
  if ( $enable != 1 and ($sched ne 'manual') ) 
  {
    print td( '&nbsp;Disabled&nbsp;' ) ;
  }
  else
  {
    # This session is enabled, and the scheduler may be running it.
    # Search for it in the active sessions stuff.
    for my $s (@ActiveSessions)
    {
      my (undef, undef, $name, undef, undef, undef, 
          undef, undef, $fname) = split( ' ', $s ) ;
      if ($sessname eq $name)
      {
        # Strip off a little bit of the file name to keep the length
        # to a minimum.
        $fname =~ s|^/var/gps|| ; 

        # There might be two sessions running on the same name (if
        # continuous logging).  If we found one already, put the two of 
        # them on separate lines.
        $status .= '<br>' if ($status) ;

        $status .= "logging to <br>&nbsp;$fname&nbsp;" ;
      }
    }
    
    # If the session isn't active ($status is blank) then we have to
    # decide if it is Pending or Completed.  
    # Continuous sessions are always running, so they normally shouldn't
    #   get here.  But if they are JUST starting up, we'll indicate they
    #   are pending.
    # Daily sessions are always either running or pending, so they are 
    #   always pending if we get to here.  
    # Once-only sessions could be pending, but only if their starttime
    #   is greater than now.
    # Manual sessions are either running or their time has run out, so
    #   we want them to display "Completed" here.  Their starttime will
    #   always be in the past and the test below will give the right
    #   result.  There is a slight chance that the session has JUST been
    #   started, but hasn't created log files yet.  Below we give a
    #   10 second margin to show "pending" while starting up.
    if (!$status)
    {
      $status =  ( ($sched eq 'daily') 
                or ($sched eq 'cont') 
                or ( ($time + 10) > $gpsTime) 
                 ) 
                 ? '&nbsp;Pending&nbsp;' 
                 : '&nbsp;Completed&nbsp;' ;
    }

    print td( $status ) ;
  }

  # If this is a manual session and it has already completed, we 
  # artificially indicate that the session is now disabled.  This lets
  # the checkbox restart the session.
  if ( ($sched eq 'manual') 
    && ($status =~ /Completed/)
     )
  {
    $enable = '0' ;
  }

  my $target= 'sessionEnableDisable( "'
              . $sessname
              . '", "'
              . ($enable ? 'disable")' : 'enable")')
              ;
  print td( checkbox( -name=>'Enabled',
                      -checked=>$enable,
                      -value=>'1',
                      -label=>'',
                      -onClick=> $target
                    ) 
          ) ;

  print "</tr>" ;
}

print '</Table>' ;
print endform() ;

# Emit the javascript function used to submit the Enable/Disable
# requests above.
print <<EOF ;
<script>

function sessionEnableDisable( sessname, state )
{
  url = '/perl-scripts/gpsadmin/startSessions.cgi?'
      + state 
      + '=' 
      + escape(sessname) ;
  parent.loadDataFrame( url ) ;
}

</script>
EOF

if (! scalar( keys(%sessData) ) ) 
{
  print '<p><b>No sessions are currently defined.</b><p>' ;
}

# Show a Timestamp:

print '<p>',timestamp(),'<p>' ;

# Get the diskspace available data.  Sadly, Perl doesn't have an
# easy way to get this via a system call.  We just run the "df" command
# and parse the output, which will look something like this:
#
#   Filesystem           1k-blocks      Used Available Use% Mounted on
#   rootfs                 6174820   3634344   2226808  62% /
#
# This is just split into a list, and the appropriate fields lifted
# from that.  'Filesystem' is field[0], 'on' is [6], etc.
my($usedSpace, $availSpace) = ( split " ", qx( df /var/gps ) )[9..10] ;


# Display the space information.
# If we are below 1Mbyte left (autodelete should kick in) make the
# space-left text bold-Red.  If we are below 100Kbytes - where logging
# won't even be started, also flash the text.  THis gives a visual
# indication of WHY if logging has stopped.
# To implement this we create an HTML string that will be inserted
# before the text to start the emphasis, and a matching string that is
# placed after the text.
my $emphasisStart = '' ;
my $emphasisEnd = '' ;
if ($availSpace < 1000)
{
  $emphasisStart = '<b><font color="red">' ;
  $emphasisEnd = '</font></b>' ;
}
if ($availSpace < 100)
{
  $emphasisStart .= "<blink>" ;
  $emphasisEnd = "</blink>".$emphasisEnd ;
}

printf "<b>Logging Space:</b> %.3f Mbytes Used.  %s%.3f Mbytes Available%s",
       $usedSpace/1000, 
       $emphasisStart,
       $availSpace/1000, 
       $emphasisEnd
       ;


my ($AutoDeleteEnable, $AutoDeleteThreshold) = ('yes', 1000000) ;

if ( open( AUTODELETE, '/etc/sysconfig/autoDelete' ) )
{
  $AutoDeleteEnable = <AUTODELETE> ;
  close AUTODELETE ;
  chomp $AutoDeleteEnable ;
  ($AutoDeleteEnable, $AutoDeleteThreshold ) 
                                     = split( ' ', $AutoDeleteEnable ) ;

  if (!$AutoDeleteThreshold)
  {
    $AutoDeleteThreshold = 1000000 ;
  }
}

print br(),br(),
      a( {href=>"javascript:parent.loadDataFrame('autoDelete.cgi')"}, 
           'AutoDelete Mode',
          
       ),
      ( ($AutoDeleteEnable =~ m/yes/)
        ? " is Enabled"  #($AutoDeleteThreshold)"
        : ' is <b>Disabled</b>.' );



if ( open( SLEEP, '/etc/sysconfig/sleepmode' ) )
{
  my $line=<SLEEP> ;
  close SLEEP ;
  chomp $line ;
  my ($Enable,$Periodic,$Hours,$Minutes) = split( ' ', $line ) ;

                   
  print p();
  print a( {href=>"javascript:parent.loadDataFrame('sleepMode.cgi')"}, 
           'Power Saving Mode' 
         ) ;
  if ($Enable eq 'yes')
  {
    print  ' is Enabled.<br>' ;
    if ($Periodic eq 'yes')
    {
      print "Periodic wakeup set to $Minutes minutes every $Hours ",
            $Hours > 1 ? "hours." : "hour." ;
    }
    else
    {
      print 'Periodic wakeup is disabled.' ;
    }
  }
  else
  {
    print ' is Disabled.' ;
  }
}

# End of section where everything is center aligned.
print '</div>' ;

# Create a one line table of "buttons" for various actions that
# might be desired from the Sessions status page.
# Note that since this page is autorefreshing every few seconds,
# using a timer in the parent window (setup at the script end below)
# you can't just link to a new location.  The timer would kick in and
# overwrite you.  Instead, use the parent function loadDataFrame, which
# cancels the timer.
print p() ;

# One of the buttons switches to editing existing sessions, and picks
# the first already defined session name.  
my $firstSessName = (sort keys %sessData)[0] ;
$firstSessName = "New Session" if (! $firstSessName) ;
$firstSessName = CGI::escape( $firstSessName ) ;

my $link= "javascript:parent.loadDataFrame('editSession.cgi?restart="
          .$firstSessName
          ."')" ;

print table( {border=>2,align=>'center',cellpadding=>5},
        Tr( 
            td( {align=>'center', width=>"25%" },
                a( {href=>
                    "javascript:parent.loadDataFrame('editSession.cgi')"
                   } , 
                   "Create a New<BR>Session" 
                 )
              ),
            td( {align=>'center', width=>"25%"},
                a( {href=>
                    $link 
                   }, 
                   "Edit<BR>Sessions" 
                 )
              ),
          )
        ) ;


print <<EOF ;

<script>
  // Reload this page every ten seconds
    parent.loadAfterDelay( window.location.href, 10000) ;
</script>

EOF


print end_html() ;

########################################################################
# end of script sessions.cgi
########################################################################
