#!/usr/bin/perl -w
# Copyright (c) 2003 Trimble Navigation Limited
# $Id: configSave.cgi,v 1.10 2003/08/05 23:13:33 tom Exp $
# $Source: /home/CVS/fireballTop/fireballFS/root/usr/local/apache/htdocs/fireball/perl-scripts/sysadmin/configSave.cgi,v $

########################################################################
# configSave.cgi
# Script for Creating new Fireball Configuration Files
# 
# Via the Web-GUI interface we can create, delete, download, upload and 
# apply "configuration files".  These are collections of Fireball
# control settings.  The files reside in /etc/sysconfig/configFiles.
#
# This script implements the actual creation of a configuration file,
# based on CGI parameters passed in from perl-scripts/configCreate.cgi.
# Currently only one is used, but others will definitely be added soon
# to control the exact contents of the configuration file.
#
# save=fname
#       Save the current settings in the named configuration file.
# 
########################################################################

use strict ;
use CGI qw/:standard/;

use lib '..' ;
use Fireball ;

########################################################################
# Begin formatting the return document.
########################################################################
startPage( 'Config Save' ) ;


# We'll build up a list of sysconfig files that have to be included
# in this array.
my @filesToInclude = () ;

if (param())
{
  #print "Parameters Received:<br>" ;
  foreach my $p (param()) 
  {
    #print "$p=",param($p),br() ;

    if ($p eq 'IOconfig' ) 
    {
      push( @filesToInclude, '/etc/sysconfig/gpssd.config' )  ;
      push( @filesToInclude, '/etc/sysconfig/met.conf' )  ;
      push( @filesToInclude,
            '/etc/sysconfig/mgetty+sendfax/mgetty.config' 
          ) ;
      push( @filesToInclude, '/etc/sysconfig/ppp/options.console' )  ;
      push( @filesToInclude, '/etc/sysconfig/ppp/options.tts.1' )  ;
      push( @filesToInclude, '/etc/sysconfig/ppp/options.tts_max0' )  ;
      push( @filesToInclude, '/etc/sysconfig/ppp/options.tts_max1' )  ;
    }

    if ($p eq 'refstation' ) 
    {
      push( @filesToInclude, '/etc/sysconfig/ref_stn.config' )  ;
    }

    if ($p eq 'sessions' ) 
    {
      push( @filesToInclude, '/etc/sysconfig/sessions' )  ;
      push( @filesToInclude, '/etc/sysconfig/sleepmode' )  ;
      push( @filesToInclude, '/etc/sysconfig/autoDelete' )  ;
    }

    if ($p eq 'stinger' ) 
    {
      push( @filesToInclude, '/etc/sysconfig/stinger.cfg' )  ;
    }

    if ($p eq 'extref' ) 
    {
      push( @filesToInclude, '/etc/sysconfig/extRefFreq' )  ;
    }

    if ($p eq 'ethernet' ) 
    {
      push( @filesToInclude, '/etc/sysconfig/interfaces' )  ;
    }

    if ($p eq 'ftp' ) 
    {
      push( @filesToInclude, '/etc/sysconfig/ftpaccess' )  ;
      push( @filesToInclude, '/etc/sysconfig/ftppasswd' )  ;
    }

    if ($p eq 'ntp' ) 
    {
      push( @filesToInclude, '/etc/sysconfig/ntp.conf' )  ;
      push( @filesToInclude, '/etc/sysconfig/step-tickers' )  ;
    }

    if ($p eq 'security' ) 
    {
      push( @filesToInclude, '/etc/sysconfig/htaccess.fileaccess' )  ;
      push( @filesToInclude, '/etc/sysconfig/htaccess.gpsadmin' )  ;
      push( @filesToInclude, '/etc/sysconfig/htaccess.sysadmin' )  ;
      push( @filesToInclude, '/etc/sysconfig/htgroups' )  ;
      push( @filesToInclude, '/etc/sysconfig/htpasswd' )  ;
    }

    if ($p eq 'everything' ) 
    {
      push( @filesToInclude, '/etc/sysconfig/sysname' )  ;
    }
  }
}


my $result ;

if (my $fname=param('fname'))
{
  # The user requested saving current configuration in a configfile.
  # $fname contains the name to give it.

  # Replace any characters we don't want in file names with underscores.
  $fname =~ s/^ +// ;
  $fname =~ s/ +$// ;
  $fname =~ tr { /} {_} ;

  $result = "Current Configuration saved in \"$fname\"." ;

  open NEWFILE, '>/tmp/newConfigFile' ;

  print NEWFILE "# ++ FBCfg v1\n" ;
  print NEWFILE "# Configuration File, Original Name = $fname\n" ;
  print NEWFILE "# created ", qx( /bin/date ), "\n";
  for my $cf (@filesToInclude)
  {
    print NEWFILE "\n## file $cf\n" ;
    open CFILE, $cf ;
    while (<CFILE>)
    {
      # We should be getting the lines one at a time, and then we'll
      # dump them into the output file.
      # Just in case the last line of the file doesn't have a newline,
      # we add it back in.  To make thie easy, we just remove them 
      # all and add an explicit newline to every line.
      chomp ;
      print NEWFILE $_, "\n" ;
    }
    close CFILE ;
    print NEWFILE "## end of file $cf\n" ;
  }
  close NEWFILE ;

  system( 
        "sudo cp /tmp/newConfigFile '/etc/sysconfig/configFiles/$fname'"
        ) ;
}
else
{
  $result = 'No configfile Name specified' ;
}



print h3( $result ),br() ;

my $returnURL = '/perl-scripts/configFiles.cgi'
              . '?message='.CGI::escape( $result )
              ;

print <<EOF ;
<Script>
  parent.loadAfterDelay( "$returnURL", 1000) ;
</Script>
EOF


print end_html() ;

########################################################################
# end of file configSave.cgi
########################################################################
