#!/usr/bin/perl -w
# Copyright (c) 2003 Trimble Navigation Limited
# $Id: ftpConfig.cgi,v 1.16.2.2 2003/10/30 22:50:38 tom Exp $
# $Source: /home/CVS/fireballTop/fireballFS/root/usr/local/apache/htdocs/fireball/perl-scripts/sysadmin/ftpConfig.cgi,v $

########################################################################
# ftpConfig.cgi
#
# Script to enable configuration of the NTP subsystem.
# Called in several forms, using CGI parameters:
#
#   ftpConfig.cgi
#       No parameters: Initialize the form with current data
#
#   ftpConfig.cgi?(full list of parameters from a form)
#       A normal submit of the information.
#
# FTP configuration is controlled by two files:
#   /etc/ftpaccess 
#       (Actually a link to /etc/sysconfig/ftpaccess)
#       This file contains the basic configuration data.  See that
#       file for extensive documentation.  We only adjust a few
#       lines in it to enable or disable specific forms of FTP.
#
#   /etc/sysconfig/ftppasswd
#       This file contains the named accounts used by FTP for 
#       password access accounts.  The file is kept separate from the
#       generic /etc/passwd file so that it can be kept as part of the
#       system configuration, and not wiped out during firmware
#       upgrades.  The file msut be merged with /etc/passwd, by using
#       the script /etc/init.d/mergeftp.  This is done every time we
#       power up (but only if needed) and by the GUI when a change is
#       made to the passwords.
########################################################################


use strict ;
use CGI qw/:standard/;

use lib '..' ;
use Fireball ;

titleWithHelp( 'FTP Setup',
               '/ftpInfo.html'
             ) ;

# If this is a CGI request then some parameters will be passed into the
# script.  We'll need to parse those and set the state(s) as requested.

my $errorMessage = '' ; # blank until an error is detected

# These values are used to hold the existing FTP setup parameters,
# and are then updated to install the new configuration values.
my $anonEnabled ;
my $namedEnabled ;
my $adminEnabled ;
my $anonDeleting ;
my $namedDeleting ;
my $namedPassword ; # encrypted
my $adminPassword ; # encrypted



if (!param())
{
  # No parameters.  Read the current setup and make that the defaults
  # for all widgets.

  # Read in the contents of /etc/ftpaccess to determine current setup
  # The variabilities here are best understood by examining
  # /etc/ftpaccess, which is well documented.  Look for "GUI WILL EDIT"
  # to find the relevant lines.
  readFTPSetup( \$anonEnabled, 
                \$namedEnabled,
                \$adminEnabled,
                \$anonDeleting,
                \$namedDeleting,
                \$namedPassword,
                \$adminPassword
              ) ;

  # Set the cgi parameters used by the GUI widgets to match what's
  # just been determined.
  param( 'anon',
         ( ! $anonEnabled 
           ? 'Disabled'
           : ( $anonDeleting
               ? 'EnabledDeleting'
               : 'Enabled'
             )
         )
       ) ;

  param( 'netrs',
         ( ! $namedEnabled 
           ? 'Disabled'
           : ( $namedDeleting
               ? 'EnabledDeleting'
               : 'Enabled'
             )
         )
       ) ;

  param( 'admin',
         ( $adminEnabled 
           ? 'Enabled'
           : 'Disabled'
         )
       ) ;
}
else
{
  # This is a submission of a new FTP setup.
  # Verify the parameters are okay...

  #print( "<pre>Parameters Received<BR>" ) ;
  #foreach my $p (param())
  #{
  #    print  "$p=", param($p), "\n" ;
  #}
  #print "</pre>" ;

  # Re-read the current state of the controls.  This is only
  # used here to get the current encrypted passwords.  The other
  # values will all be overwritten from the form's buttons.
  readFTPSetup( \$anonEnabled, 
                \$namedEnabled,
                \$adminEnabled,
                \$anonDeleting,
                \$namedDeleting,
                \$namedPassword,
                \$adminPassword
              ) ;

  my $netrspw = '' ; # Will remain blank if nothing specified by user
  my $adminpw = '' ; # Will remain blank if nothing specified by user

  my $pw1 ;
  my $pw2 ;

  if ( !($pw1 = param( 'netrspw1' )) ) {$pw1 = ''} ;
  if ( !($pw2 = param( 'netrspw2' )) ) {$pw2 = ''} ;

  if ( $pw1 or $pw2 )
  {
    if ($pw1 ne $pw2)
    {
      $errorMessage = 'netrsFTP Passwords must match.' ;
      goto PARAMDONE ;
    }
    $netrspw = $pw1 ;
  }
  
 
  if ( !($pw1 = param( 'adminpw1' )) ) {$pw1 = ''} ;
  if ( !($pw2 = param( 'adminpw2' )) ) {$pw2 = ''} ;

  if ( ( $pw1 or $pw2 )
    && ( param( 'admin' ) eq 'Enabled' ) 
     )
  {
    if ($pw1 ne $pw2)
    {
      $errorMessage = 'adminFTP Passwords must match' ;
      goto PARAMDONE ;
    }
    $adminpw = $pw1 ;
  }
  

  # These are the new states of the controls.
  $anonEnabled   = ( param( 'anon' ) ne 'Disabled' ) ;
  $anonDeleting  = ( param( 'anon' ) eq 'EnabledDeleting' ) ;
  $namedEnabled  = ( param( 'netrs' ) ne 'Disabled' ) ;
  $namedDeleting = ( param( 'netrs' ) eq 'EnabledDeleting' ) ;
  $adminEnabled  = ( param( 'admin' ) ne 'Disabled' ) ;


  # Check if the passwords aren't defined where they are needed.
  if ( $namedEnabled and !$namedPassword and !$netrspw ) 
  {
    $errorMessage = 'netrsFTP needs a password.' ;
    goto PARAMDONE ;
  }

  if ( $adminEnabled and !$adminPassword and !$adminpw ) 
  {
    $errorMessage = 'adminFTP needs a password.' ;
    goto PARAMDONE ;
  }
 

  # Update the /etc/sysconfig/ftppasswd file with any new
  # passwords.  We feed in the old encrypted passwords (if they existed)
  # as the first two parameters.  These will be re-installed unless
  # new clear-text passwords are sent in the third or fourth
  # parameters.  If those were entered by the user, then they will
  # be encrypted and overwrite the original settings.
  # Note that if an old password didn't exist (is blank) and no new
  # password is entered (is blank) then the password file will 
  # not have a line for that account.  This can't currently be forced
  # from the GUI.
  generateFtppasswd( $namedPassword, 
                     $adminPassword,
                     $netrspw, 
                     $adminpw 
                   ) ;

 
  # Update the ftpaccess config file.
  # Also Restarts the FTP daemon program.
  regenerateFtpAccess( $anonEnabled,
                       $anonDeleting,
                       $namedEnabled,
                       $namedDeleting,
                       $adminEnabled
                     ) ;

  print h2( 'New FTP setup accepted.' ) ;
  print h2( 'Restarting FTP Server.' ) ;

  print <<EOF ;
  <Script>
    parent.loadAfterDelay( "/perl-scripts/ftp.cgi", 2000) ;
  </Script>
EOF

  goto EXIT ;

}
PARAMDONE:

# Done processing parameters (if any) and nobody forced an exit.
# So we aren't done and have to put up the HTML form to request
# user input.

print startform( -name=>"theForm" ) ;

print '<table border=0><tr><td>' ;

# ANONYMOUS FTP CONTROLS
print b("Anonymous FTP") ;

# Create three buttons.
my @buttons=radio_group( -name=>'anon',
                         -values=>[ 'EnabledDeleting',
                                    'Enabled', 
                                    'Disabled'
                                  ],
                         -nolabels=>1,
                         -default=>"Enabled",
                       ) ;


print table( {width=>"100%", border=>1},

             Tr( td( $buttons[0] ), 
                 td( "<b>Enabled  with Delete</b><br>
                      Anyone using the names 'anonymous' or 'ftp' can 
                      use FTP to access logged data files.  
                      No password is required.
                      <br>Anonymous user <em>can</em> delete files.
                     "
                   ) 
               ) ,

             Tr( td( $buttons[1] ), 
                 td( "<b>Enabled without Delete</b><br>
                      Anyone using the names 'anonymous' or 'ftp' can 
                      use FTP to access logged data files.  
                      No password is required.
                      <br>Anonymous user <em>cannot</em> delete files.
                     "
                   ) 
               ) ,

             Tr( td( $buttons[2]),
                 td( "<b>Disabled</b><br>
                      Anonymous FTP is not available." ) 
               ) 
           ) ;

print '</td><td>&nbsp;</td></tr>' ;

# Named FTP controls

print Tr( td( br(), b("Named FTP") )  ) ;

print '<tr><td>' ;

# Create three buttons.
@buttons=radio_group( -name=>'netrs',
                      -values=>[ 'EnabledDeleting', 
                                 'Enabled', 
                                 'Disabled'
                               ],
                      -nolabels=>1,
                      -default=>"Enabled",
                    ) ;


print table( {width=>"100%", border=>1},

             Tr( td( $buttons[0] ), 
                 td( "<b>Enabled  with Delete</b><br>
                      User 'netrsFTP' can access logged data files using
                      FTP and the proper password. 
                      <br>User 'netrsFTP' <em>can</em> delete files.
                     "
                   ) 
               ) ,

             Tr( td( $buttons[1] ), 
                 td( "<b>Enabled without Delete</b><br>
                      User 'netrsFTP' can access logged data files using
                      FTP and the proper password. 
                      <br>User 'netrsFTP' <em>cannot</em> delete files.
                     "
                   ) 
               ) ,

             Tr( td( $buttons[2]),
                 td( "<b>Disabled</b><br>
                      User 'netrsFTP' cannot use FTP." ) 
               ) 
           ) ;

print '</td><td width="1"></td><td valign="top">' ;


print table( {border=>'0'},
             Tr( th( {-align=>'right'}, 'netrsFTP Password' ),
                 td( password_field( 'netrspw1', '', 12, 12 ) )
               ),

             Tr( th( {-align=>'right'}, 'Re-enter Password' ),
                 td( password_field( 'netrspw2', '', 12, 12 ) )
               ),

           ) ;

print <<EOF ;
<div align="right">
<font size="-1"'>
If both fields are left blank, the
<br> password will not be changed.</font></div></td>
</tr>
<tr>
<td>
EOF



# adminFTP controls

print Tr( td( br(), b("Administration FTP") )  ) ;
print '<tr><td valign="top">' ;

# Create three buttons.
 @buttons=radio_group( -name=>'admin',
                       -values=>[ 'Enabled', 'Disabled' ],
                       -nolabels=>1,
                       -default=>"Enabled",
                     ) ;



print table( {width=>"100%", border=>1},

             Tr( td( $buttons[0] ), 
                 td( "<b>Enabled</b><br>
                      User 'adminFTP' can upload or delete firmware 
                      files, using FTP and the proper password. 
                     "
                   ) 
               ) ,

             Tr( td( $buttons[1] ), 
                 td( "<b>Disabled</b><br>
                      User 'adminFTP' cannot use FTP.
                     "
                   ) 
               ) ,
           ) ;

print '</td><td></td><td valign="top">' ;

print table( 
             #Tr( th( '&nbsp;' ),
             #  ) ,

             Tr( th( {-align=>'right'}, 'adminFTP Password' ),
                 td( password_field( 'adminpw1', '', 12, 12 ) )
               ) ,

             Tr( th( {-align=>'right'}, 'Re-enter Password' ),
                 td( password_field( 'adminpw2', '', 12, 12 ) )
               ) 
           ) ;

print <<EOF ;
<div align="right">
<font size="-1"'>
If both fields are left blank, the
<br> password will not be changed.</font></div></td>
</tr>
</table>
EOF



print p(),button( -name=>'OK', 
              -value=>'OK',
              -onClick=>'parent.submitForm( document.theForm, 
                              "/perl-scripts/sysadmin/ftpConfig.cgi")'
            ) ;
print "&nbsp;" ;
print button( -name=>'Cancel', 
              -value=>'Cancel',
              -onClick=>
                       'parent.loadDataFrame( "/perl-scripts/ftp.cgi" )'
            ) ;


print endform() ;

# If an error message was found earlier during CGI processing,
# post an alert message to the user.  Also display the message on the
# bottom of the page, in case the javascript alert system doesn't work.
if ( $errorMessage )
{
  print << "EOF" ;
  <Script> 
  alert( "$errorMessage" ) ;
  </Script>
EOF

  print b( $errorMessage ) ;
}

EXIT:
print end_html() ;

########################################################################
# End of file ftpConfig.cgi
########################################################################
